@@ -17,7 +17,7 @@ class TurbineAnalyzer : public BoomAnalyzer
     public:
         Q_INVOKABLE TurbineAnalyzer( QWidget *parent ) : BoomAnalyzer( parent ) {}
 
-        void analyze( QPainter& p, const Scope& );
+        void analyze( QPainter& p, const Scope&, bool new_frame);
 
         static const char* kName;
 };
