@@ -22,6 +22,7 @@
 #include <QStringList>
 #include <QTimer>
 #include <QThread>
+#include <QUrl>
 
 const int DeleteFiles::kBatchSize = 50;
 
@@ -60,7 +61,7 @@ void DeleteFiles::Start(const QStringList& filenames) {
   SongList songs;
   foreach (const QString& filename, filenames) {
     Song song;
-    song.set_filename(filename);
+    song.set_url(QUrl::fromLocalFile(filename));
     songs << song;
   }
 
