@@ -23,6 +23,8 @@
 class GlobalShortcuts;
 
 class GlobalShortcutBackend : public QObject {
+  Q_OBJECT
+
 public:
   GlobalShortcutBackend(GlobalShortcuts* parent = 0);
   virtual ~GlobalShortcutBackend() {}
@@ -31,7 +33,9 @@ public:
 
   bool Register();
   void Unregister();
-  void Reregister();
+
+signals:
+  void RegisterFinished(bool success);
 
 protected:
   virtual bool DoRegister() = 0;
