@@ -15,38 +15,46 @@
    along with Clementine.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "organiseformat.h"
-#include "core/timeconstants.h"
+#include "core/organiseformat.h"
 
 #include <QApplication>
 #include <QPalette>
 #include <QUrl>
 
+#include "core/timeconstants.h"
+
 const char* OrganiseFormat::kTagPattern = "\\%([a-zA-Z]*)";
 const char* OrganiseFormat::kBlockPattern = "\\{([^{}]+)\\}";
 const QStringList OrganiseFormat::kKnownTags = QStringList()
     << "title" << "album" << "artist" << "artistinitial" << "albumartist"
     << "composer" << "track" << "disc" << "bpm" << "year" << "genre"
-    << "comment" << "length" << "bitrate" << "samplerate" << "extension";
+    << "comment" << "length" << "bitrate" << "samplerate" << "extension"
+    << "performer" << "grouping";
 
 // From http://en.wikipedia.org/wiki/8.3_filename#Directory_table
 const char* OrganiseFormat::kInvalidFatCharacters = "\"*/\\:<>?|";
-const int OrganiseFormat::kInvalidFatCharactersCount = strlen(OrganiseFormat::kInvalidFatCharacters);
+const int OrganiseFormat::kInvalidFatCharactersCount =
+    strlen(OrganiseFormat::kInvalidFatCharacters);
 
-const QRgb OrganiseFormat::SyntaxHighlighter::kValidTagColorLight = qRgb(64, 64, 255);
-const QRgb OrganiseFormat::SyntaxHighlighter::kInvalidTagColorLight = qRgb(255, 64, 64);
-const QRgb OrganiseFormat::SyntaxHighlighter::kBlockColorLight = qRgb(230, 230, 230);
+const QRgb OrganiseFormat::SyntaxHighlighter::kValidTagColorLight =
+    qRgb(64, 64, 255);
+const QRgb OrganiseFormat::SyntaxHighlighter::kInvalidTagColorLight =
+    qRgb(255, 64, 64);
+const QRgb OrganiseFormat::SyntaxHighlighter::kBlockColorLight =
+    qRgb(230, 230, 230);
 
-const QRgb OrganiseFormat::SyntaxHighlighter::kValidTagColorDark = qRgb(128, 128, 255);
-const QRgb OrganiseFormat::SyntaxHighlighter::kInvalidTagColorDark = qRgb(255, 128, 128);
-const QRgb OrganiseFormat::SyntaxHighlighter::kBlockColorDark = qRgb(64, 64, 64);
+const QRgb OrganiseFormat::SyntaxHighlighter::kValidTagColorDark =
+    qRgb(128, 128, 255);
+const QRgb OrganiseFormat::SyntaxHighlighter::kInvalidTagColorDark =
+    qRgb(255, 128, 128);
+const QRgb OrganiseFormat::SyntaxHighlighter::kBlockColorDark =
+    qRgb(64, 64, 64);
 
 OrganiseFormat::OrganiseFormat(const QString &format)
   : format_(format),
     replace_non_ascii_(false),
     replace_spaces_(false),
-    replace_the_(false)
-{
+    replace_the_(false) {
 }
 
 void OrganiseFormat::set_format(const QString &v) {
@@ -70,11 +78,11 @@ QString OrganiseFormat::GetFilenameForSong(const Song &song) const {
 
   if (replace_non_ascii_) {
     QString stripped;
-    for (int i=0 ; i<filename.length() ; ++i) {
+    for (int i = 0 ; i < filename.length(); ++i) {
       const QCharRef c = filename[i];
-      if (c < 128)
+      if (c < 128) {
         stripped.append(c);
-      else {
+      } else {
         const QString decomposition = c.decomposition();
         if (!decomposition.isEmpty() && decomposition[0] < 128)
           stripped.append(decomposition[0]);
@@ -131,22 +139,29 @@ QString OrganiseFormat::TagValue(const QString &tag, const Song &song) const {
   else if (tag == "album")       value = song.album();
   else if (tag == "artist")      value = song.artist();
   else if (tag == "composer")    value = song.composer();
+  else if (tag == "performer")   value = song.performer();
+  else if (tag == "grouping")    value = song.grouping();
   else if (tag == "genre")       value = song.genre();
   else if (tag == "comment")     value = song.comment();
   else if (tag == "year")        value = QString::number(song.year());
   else if (tag == "track")       value = QString::number(song.track());
   else if (tag == "disc")        value = QString::number(song.disc());
   else if (tag == "bpm")         value = QString::number(song.bpm());
-  else if (tag == "length")      value = QString::number(song.length_nanosec() / kNsecPerSec);
+  else if (tag == "length")      value =
+      QString::number(song.length_nanosec() / kNsecPerSec);
   else if (tag == "bitrate")     value = QString::number(song.bitrate());
   else if (tag == "samplerate")  value = QString::number(song.samplerate());
-  else if (tag == "extension")   value = song.url().toLocalFile().section('.', -1, -1);
+  else if (tag == "extension")   value =
+      song.url().toLocalFile().section('.', -1, -1);
   else if (tag == "artistinitial") {
     value = song.effective_albumartist().trimmed();
+    if (replace_the_ && !value.isEmpty())
+      value.replace(QRegExp("^the\\s+", Qt::CaseInsensitive), "");
     if (!value.isEmpty()) value = value[0].toUpper();
-  }
-  else if (tag == "albumartist") {
-    value = song.is_compilation() ? "Various Artists" : song.effective_albumartist();
+  } else if (tag == "albumartist") {
+    value = song.is_compilation()
+        ?  "Various Artists"
+        : song.effective_albumartist();
   }
 
   if (replace_the_ && (tag == "artist" || tag == "albumartist"))
@@ -160,7 +175,7 @@ QString OrganiseFormat::TagValue(const QString &tag, const Song &song) const {
     value.prepend('0');
 
   // Replace characters that really shouldn't be in paths
-  for (int i=0 ; i<kInvalidFatCharactersCount ; ++i) {
+  for (int i = 0 ; i < kInvalidFatCharactersCount; ++i) {
     value.replace(kInvalidFatCharacters[i], '_');
   }
 
@@ -171,16 +186,17 @@ QString OrganiseFormat::TagValue(const QString &tag, const Song &song) const {
 OrganiseFormat::Validator::Validator(QObject *parent)
   : QValidator(parent) {}
 
-QValidator::State OrganiseFormat::Validator::validate(QString& input, int&) const {
+QValidator::State OrganiseFormat::Validator::validate(
+    QString& input, int&) const {
   QRegExp tag_regexp(kTagPattern);
 
   // Make sure all the blocks match up
   int block_level = 0;
-  for (int i=0 ; i<input.length() ; ++i) {
+  for (int i = 0 ; i < input.length(); ++i) {
     if (input[i] == '{')
-      block_level ++;
+      block_level++;
     else if (input[i] == '}')
-      block_level --;
+      block_level--;
 
     if (block_level < 0 || block_level > 1)
       return QValidator::Invalid;
@@ -212,10 +228,12 @@ OrganiseFormat::SyntaxHighlighter::SyntaxHighlighter(QTextDocument* parent)
   : QSyntaxHighlighter(parent) {}
 
 void OrganiseFormat::SyntaxHighlighter::highlightBlock(const QString& text) {
-  const bool light = QApplication::palette().color(QPalette::Base).value() > 128;
+  const bool light =
+      QApplication::palette().color(QPalette::Base).value() > 128;
   const QRgb block_color = light ? kBlockColorLight : kBlockColorDark;
   const QRgb valid_tag_color = light ? kValidTagColorLight : kValidTagColorDark;
-  const QRgb invalid_tag_color = light ? kInvalidTagColorLight : kInvalidTagColorDark;
+  const QRgb invalid_tag_color =
+      light ? kInvalidTagColorLight : kInvalidTagColorDark;
 
   QRegExp tag_regexp(kTagPattern);
   QRegExp block_regexp(kBlockPattern);
@@ -238,8 +256,10 @@ void OrganiseFormat::SyntaxHighlighter::highlightBlock(const QString& text) {
   pos = 0;
   while ((pos = tag_regexp.indexIn(text, pos)) != -1) {
     QTextCharFormat f = format(pos);
-    f.setForeground(QColor(OrganiseFormat::kKnownTags.contains(tag_regexp.cap(1))
-                           ? valid_tag_color : invalid_tag_color));
+    f.setForeground(
+        QColor(OrganiseFormat::kKnownTags.contains(tag_regexp.cap(1))
+               ? valid_tag_color
+               : invalid_tag_color));
 
     setFormat(pos, tag_regexp.matchedLength(), f);
     pos += tag_regexp.matchedLength();
