@@ -16,6 +16,7 @@
 */
 
 #include "scopedtransaction.h"
+#include "core/logging.h"
 
 #include <QSqlDatabase>
 #include <QtDebug>
@@ -29,14 +30,14 @@ ScopedTransaction::ScopedTransaction(QSqlDatabase* db)
 
 ScopedTransaction::~ScopedTransaction() {
   if (pending_) {
-    qDebug() << __PRETTY_FUNCTION__ << "Rolling back transaction";
+    qLog(Warning) << "Rolling back transaction";
     db_->rollback();
   }
 }
 
 void ScopedTransaction::Commit() {
   if (!pending_) {
-    qWarning() << "Tried to commit a ScopedTransaction twice";
+    qLog(Warning) << "Tried to commit a ScopedTransaction twice";
     return;
   }
 
