@@ -15,32 +15,38 @@
    along with Clementine.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "timeconstants.h"
 #include "utilities.h"
-#include "core/logging.h"
 
-#include "sha2.h"
+#include <stdlib.h>
+
+#include <boost/scoped_array.hpp>
 
 #include <QApplication>
 #include <QDateTime>
 #include <QDesktopServices>
 #include <QDir>
 #include <QIODevice>
+#include <QMetaEnum>
 #include <QMouseEvent>
 #include <QStringList>
 #include <QTcpServer>
+#include <QtDebug>
 #include <QTemporaryFile>
+#include <QtGlobal>
 #include <QUrl>
 #include <QWidget>
 #include <QXmlStreamReader>
-#include <QtDebug>
-#include <QtGlobal>
-#include <QMetaEnum>
+
+#include "core/logging.h"
+#include "timeconstants.h"
+
+#include "sha2.h"
 
 #if defined(Q_OS_UNIX)
 #  include <sys/statvfs.h>
 #elif defined(Q_OS_WIN32)
 #  include <windows.h>
+#  include <QProcess>
 #endif
 
 #ifdef Q_OS_LINUX
@@ -57,10 +63,9 @@
 #  include "CoreServices/CoreServices.h"
 #  include "IOKit/ps/IOPowerSources.h"
 #  include "IOKit/ps/IOPSKeys.h"
+#  include <QProcess>
 #endif
 
-#include <boost/scoped_array.hpp>
-
 namespace Utilities {
 
 static QString tr(const char* str) {
@@ -123,7 +128,7 @@ QString Ago(int seconds_since_epoch, const QLocale& locale) {
   if (days_ago <= 7)
     return tr("%1 days ago").arg(days_ago);
 
-  return then.date().toString(locale.dateFormat());
+  return then.date().toString(locale.dateFormat(QLocale::ShortFormat));
 }
 
 QString PrettyFutureDate(const QDate& date) {
@@ -207,6 +212,17 @@ QString MakeTempDir(const QString template_name) {
   return path;
 }
 
+QString GetTemporaryFileName() {
+  QString file;
+  {
+    QTemporaryFile tempfile;
+    tempfile.open();
+    file = tempfile.fileName();
+  }
+
+  return file;
+}
+
 void RemoveRecursive(const QString& path) {
   QDir dir(path);
   foreach (const QString& child, dir.entryList(QDir::NoDotAndDotDot | QDir::Dirs | QDir::Hidden))
@@ -290,14 +306,28 @@ QString GetConfigPath(ConfigPath config) {
     }
     break;
 
+    case Path_CacheRoot: {
+      #if defined(Q_OS_UNIX) && !defined(Q_OS_DARWIN)
+        char* xdg = getenv("XDG_CACHE_HOME");
+        if (!xdg || !*xdg) {
+          return QString("%1/.cache/%2").arg(QDir::homePath(), QCoreApplication::organizationName());
+        } else {
+          return QString("%1/%2").arg(xdg, QCoreApplication::organizationName());
+        }
+      #else
+        return GetConfigPath(Path_Root);
+      #endif
+    }
+    break;
+
     case Path_AlbumCovers:
       return GetConfigPath(Path_Root) + "/albumcovers";
 
     case Path_NetworkCache:
-      return GetConfigPath(Path_Root) + "/networkcache";
-      
+      return GetConfigPath(Path_CacheRoot) + "/networkcache";
+
     case Path_MoodbarCache:
-      return GetConfigPath(Path_Root) + "/moodbarcache";
+      return GetConfigPath(Path_CacheRoot) + "/moodbarcache";
 
     case Path_GstreamerRegistry:
       return GetConfigPath(Path_Root) +
@@ -325,8 +355,20 @@ qint32 GetMacVersion() {
   Gestalt(gestaltSystemVersionMinor, &minor_version);
   return minor_version;
 }
+
+// Better than openUrl(dirname(path)) - also highlights file at path
+void RevealFileInFinder(QString const& path) {
+  QProcess::execute("/usr/bin/open", QStringList() << "-R" << path);
+}
 #endif  // Q_OS_DARWIN
 
+#ifdef Q_OS_WIN
+void ShowFileInExplorer(QString const& path) {
+  QProcess::execute("explorer.exe", QStringList() << "/select,"
+                                                  << QDir::toNativeSeparators(path));
+}
+#endif
+
 void OpenInFileBrowser(const QList<QUrl>& urls) {
   QSet<QString> dirs;
 
@@ -340,12 +382,19 @@ void OpenInFileBrowser(const QList<QUrl>& urls) {
       continue;
 
     const QString directory = QFileInfo(path).dir().path();
-
     if (dirs.contains(directory))
       continue;
     dirs.insert(directory);
-
+    qLog(Debug) << path;
+#ifdef Q_OS_DARWIN
+    // revealing multiple files in the finder only opens one window,
+    // so it also makes sense to reveal at most one per directory
+    RevealFileInFinder(path);
+#elif defined(Q_OS_WIN32)
+    ShowFileInExplorer(path);
+#else
     QDesktopServices::openUrl(QUrl::fromLocalFile(directory));
+#endif
   }
 }
 
@@ -365,6 +414,11 @@ QByteArray Hmac(const QByteArray& key, const QByteArray& data, HashFunction meth
                                     QCryptographicHash::hash(inner_padding + data,
                                                              QCryptographicHash::Md5),
                                     QCryptographicHash::Md5);
+  } else if (Sha1_Algo == method) {
+    return QCryptographicHash::hash(outer_padding +
+                                    QCryptographicHash::hash(inner_padding + data,
+                                                             QCryptographicHash::Sha1),
+                                    QCryptographicHash::Sha1);
   } else { // Sha256_Algo, currently default
     return Sha256(outer_padding + Sha256(inner_padding + data));
   }
@@ -378,6 +432,10 @@ QByteArray HmacMd5(const QByteArray& key, const QByteArray& data) {
   return Hmac(key, data, Md5_Algo);
 }
 
+QByteArray HmacSha1(const QByteArray& key, const QByteArray& data) {
+  return Hmac(key, data, Sha1_Algo);
+}
+
 QByteArray Sha256(const QByteArray& data) {
   SHA256_CTX context;
   SHA256_Init(&context);
@@ -520,13 +578,9 @@ bool IsLaptop() {
   }
 
   return !(status.BatteryFlag & 128); // 128 = no system battery
-#endif
-
-#ifdef Q_OS_LINUX
+#elif defined(Q_OS_LINUX)
   return !QDir("/proc/acpi/battery").entryList(QDir::Dirs | QDir::NoDotAndDotDot).isEmpty();
-#endif
-
-#ifdef Q_OS_MAC
+#elif defined(Q_OS_MAC)
   ScopedCFTypeRef<CFTypeRef> power_sources(IOPSCopyPowerSourcesInfo());
   ScopedCFTypeRef<CFArrayRef> power_source_list(
       IOPSCopyPowerSourcesList(power_sources.get()));
@@ -540,9 +594,25 @@ bool IsLaptop() {
     }
   }
   return false;
+#else
+  return false;
 #endif
 }
 
+QString SystemLanguageName() {
+#if QT_VERSION >= 0x040800
+  QString system_language = QLocale::system().uiLanguages().empty() ?
+      QLocale::system().name() : QLocale::system().uiLanguages().first();
+  // uiLanguages returns strings with "-" as separators for language/region;
+  // however QTranslator needs "_" separators
+  system_language.replace("-", "_");
+#else
+  QString system_language = QLocale::system().name();
+#endif
+
+  return system_language;
+}
+
 }  // namespace Utilities
 
 
