@@ -49,6 +49,7 @@ namespace Utilities {
   quint64 FileSystemFreeSpace(const QString& path);
 
   QString MakeTempDir(const QString template_name = QString());
+  QString GetTemporaryFileName();
   void RemoveRecursive(const QString& path);
   bool CopyRecursive(const QString& source, const QString& destination);
   bool Copy(QIODevice* source, QIODevice* destination);
@@ -58,10 +59,12 @@ namespace Utilities {
   enum HashFunction {
     Md5_Algo,
     Sha256_Algo,
+    Sha1_Algo,
   };
   QByteArray Hmac(const QByteArray& key, const QByteArray& data, HashFunction algo);
   QByteArray HmacMd5(const QByteArray& key, const QByteArray& data);
   QByteArray HmacSha256(const QByteArray& key, const QByteArray& data);
+  QByteArray HmacSha1(const QByteArray& key, const QByteArray& data);
   QByteArray Sha256(const QByteArray& data);
 
 
@@ -110,6 +113,7 @@ namespace Utilities {
     Path_DefaultMusicLibrary,
     Path_LocalSpotifyBlob,
     Path_MoodbarCache,
+    Path_CacheRoot,
   };
   QString GetConfigPath(ConfigPath config);
 
@@ -135,6 +139,8 @@ namespace Utilities {
 
   // Returns true if this machine has a battery.
   bool IsLaptop();
+
+  QString SystemLanguageName();
 }
 
 class ScopedWCharArray {
