@@ -16,12 +16,6 @@
 */
 
 #include "albumcoverloader.h"
-#include "config.h"
-#include "core/logging.h"
-#include "core/network.h"
-#include "core/tagreaderclient.h"
-#include "core/utilities.h"
-#include "internet/internetmodel.h"
 
 #include <QPainter>
 #include <QDir>
@@ -29,9 +23,15 @@
 #include <QUrl>
 #include <QNetworkReply>
 
-#ifdef HAVE_SPOTIFY
-# include "internet/spotifyservice.h"
-#endif
+#include "config.h"
+#include "core/closure.h"
+#include "core/logging.h"
+#include "core/network.h"
+#include "core/tagreaderclient.h"
+#include "core/utilities.h"
+#include "internet/internetmodel.h"
+#include "internet/spotifyservice.h"
+
 
 
 AlbumCoverLoader::AlbumCoverLoader(QObject* parent)
@@ -162,35 +162,33 @@ AlbumCoverLoader::TryLoadResult AlbumCoverLoader::TryLoadImage(
   }
 
   if (filename.toLower().startsWith("http://")) {
-    QNetworkReply* reply = network_->get(QNetworkRequest(filename));
-    connect(reply, SIGNAL(finished()), SLOT(RemoteFetchFinished()));
+    QUrl url(filename);
+    QNetworkReply* reply = network_->get(QNetworkRequest(url));
+    NewClosure(reply, SIGNAL(finished()), this,
+               SLOT(RemoteFetchFinished(QNetworkReply*)), reply);
 
     remote_tasks_.insert(reply, task);
     return TryLoadResult(true, false, QImage());
   } else if (filename.toLower().startsWith("spotify://image/")) {
     // HACK: we should add generic image URL handlers
-    #ifdef HAVE_SPOTIFY
-      SpotifyService* spotify = InternetModel::Service<SpotifyService>();
+    SpotifyService* spotify = InternetModel::Service<SpotifyService>();
 
-      if (!connected_spotify_) {
-        connect(spotify, SIGNAL(ImageLoaded(QString,QImage)),
-                SLOT(SpotifyImageLoaded(QString,QImage)));
-        connected_spotify_ = true;
-      }
+    if (!connected_spotify_) {
+      connect(spotify, SIGNAL(ImageLoaded(QString,QImage)),
+              SLOT(SpotifyImageLoaded(QString,QImage)));
+      connected_spotify_ = true;
+    }
 
-      QString id = QUrl(filename).path();
-      if (id.startsWith('/')) {
-        id.remove(0, 1);
-      }
-      remote_spotify_tasks_.insert(id, task);
+    QString id = QUrl(filename).path();
+    if (id.startsWith('/')) {
+      id.remove(0, 1);
+    }
+    remote_spotify_tasks_.insert(id, task);
 
-      // Need to schedule this in the spotify service's thread
-      QMetaObject::invokeMethod(spotify, "LoadImage", Qt::QueuedConnection,
-                                Q_ARG(QString, id));
-      return TryLoadResult(true, false, QImage());
-    #else
-      return TryLoadResult(false, false, QImage());
-    #endif
+    // Need to schedule this in the spotify service's thread
+    QMetaObject::invokeMethod(spotify, "LoadImage", Qt::QueuedConnection,
+                              Q_ARG(QString, id));
+    return TryLoadResult(true, false, QImage());
   }
 
   QImage image(filename);
@@ -208,10 +206,7 @@ void AlbumCoverLoader::SpotifyImageLoaded(const QString& id, const QImage& image
   emit ImageLoaded(task.id, scaled, image);
 }
 
-void AlbumCoverLoader::RemoteFetchFinished() {
-  QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
-  if (!reply)
-    return;
+void AlbumCoverLoader::RemoteFetchFinished(QNetworkReply* reply) {
   reply->deleteLater();
 
   Task task = remote_tasks_.take(reply);
@@ -225,7 +220,8 @@ void AlbumCoverLoader::RemoteFetchFinished() {
     QNetworkRequest request = reply->request();
     request.setUrl(redirect.toUrl());
     QNetworkReply* redirected_reply = network_->get(request);
-    connect(redirected_reply, SIGNAL(finished()), SLOT(RemoteFetchFinished()));
+    NewClosure(redirected_reply, SIGNAL(finished()), this,
+               SLOT(RemoteFetchFinished(QNetworkReply*)), redirected_reply);
 
     remote_tasks_.insert(redirected_reply, task);
     return;
