@@ -1,14 +1,16 @@
 #include "kittenloader.h"
-#include "core/network.h"
 
 #include <QNetworkReply>
 #include <QNetworkRequest>
 #include <QXmlStreamReader>
 
+#include "core/closure.h"
+#include "core/network.h"
+
 const char* KittenLoader::kFlickrKittenUrl =
     "http://api.flickr.com/services/rest/"
     "?method=flickr.photos.search"
-    "&api_key=b60e59d549d314c98ddb1dc06008a2c4"
+    "&api_key=808b52887b3cc7fe098abd62f6ed1745"
     "&tags=kitten"
     "&sort=random"
     "&safe_search=1"
@@ -48,12 +50,11 @@ quint64 KittenLoader::LoadKitten(const AlbumCoverLoaderOptions& options) {
 void KittenLoader::FetchMoreKittens() {
   QNetworkRequest req = QNetworkRequest(QUrl(kFlickrKittenUrl));
   QNetworkReply* reply = network_->get(req);
-  connect(reply, SIGNAL(finished()), SLOT(KittensRetrieved()));
+  NewClosure(reply, SIGNAL(finished()), this,
+             SLOT(KittensRetrieved(QNetworkReply*)), reply);
 }
 
-void KittenLoader::KittensRetrieved() {
-  QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
-  Q_ASSERT(reply);
+void KittenLoader::KittensRetrieved(QNetworkReply* reply) {
   reply->deleteLater();
 
   QXmlStreamReader reader(reply);
