@@ -15,10 +15,22 @@
    along with Clementine.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "devicemanager.h"
+
+#include <boost/bind.hpp>
+
+#include <QApplication>
+#include <QDir>
+#include <QIcon>
+#include <QMessageBox>
+#include <QPainter>
+#include <QPushButton>
+#include <QSortFilterProxyModel>
+#include <QUrl>
+
 #include "config.h"
 #include "devicedatabasebackend.h"
 #include "devicekitlister.h"
-#include "devicemanager.h"
 #include "devicestatefiltermodel.h"
 #include "filesystemdevice.h"
 #include "core/application.h"
@@ -38,37 +50,17 @@
 #ifdef Q_OS_DARWIN
 #  include "macdevicelister.h"
 #endif
-#ifdef Q_OS_WIN32
-#  ifdef HAVE_SAC
-#    include "wmdmlister.h"
-#    include "wmdmdevice.h"
-#  endif
-#endif
 #ifdef HAVE_LIBGPOD
 #  include "gpoddevice.h"
 #endif
 #ifdef HAVE_GIO
 #  include "giolister.h"
 #endif
-#ifdef HAVE_IMOBILEDEVICE
-#  include "afcdevice.h"
-#  include "ilister.h"
-#endif
 #ifdef HAVE_LIBMTP
 #  include "mtpdevice.h"
 #endif
 
-#include <QApplication>
-#include <QDir>
-#include <QIcon>
-#include <QMessageBox>
-#include <QPainter>
-#include <QPushButton>
-#include <QSortFilterProxyModel>
-#include <QUrl>
-
-#include <tr1/functional>
-using std::tr1::bind;
+using boost::bind;
 
 const int DeviceManager::kDeviceIconSize = 32;
 const int DeviceManager::kDeviceIconOverlaySize = 16;
@@ -206,14 +198,6 @@ DeviceManager::DeviceManager(Application* app, QObject *parent)
 #ifdef Q_OS_DARWIN
   AddLister(new MacDeviceLister);
 #endif
-#if defined(Q_OS_WIN32) && defined(HAVE_SAC)
-  AddLister(new WmdmLister);
-  AddDeviceClass<WmdmDevice>();
-#endif
-#ifdef HAVE_IMOBILEDEVICE
-  AddLister(new iLister);
-  AddDeviceClass<AfcDevice>();
-#endif
 
   AddDeviceClass<FilesystemDevice>();
 
