@@ -19,7 +19,6 @@
 #define FILESYSTEMDEVICE_H
 
 #include "connecteddevice.h"
-#include "core/backgroundthread.h"
 #include "core/filesystemmusicstorage.h"
 
 class DeviceManager;
@@ -32,6 +31,7 @@ public:
   Q_INVOKABLE FilesystemDevice(
       const QUrl& url, DeviceLister* lister,
       const QString& unique_id, DeviceManager* manager,
+      Application* app,
       int database_id, bool first_time);
   ~FilesystemDevice();
 
@@ -40,7 +40,8 @@ public:
   static QStringList url_schemes() { return QStringList() << "file"; }
 
 private:
-  BackgroundThread<LibraryWatcher>* watcher_;
+  LibraryWatcher* watcher_;
+  QThread* watcher_thread_;
 };
 
 #endif // FILESYSTEMDEVICE_H
