@@ -17,6 +17,7 @@
 
 #include "connecteddevice.h"
 #include "gpodloader.h"
+#include "core/logging.h"
 #include "core/song.h"
 #include "core/taskmanager.h"
 #include "library/librarybackend.h"
@@ -53,7 +54,7 @@ void GPodLoader::LoadDatabase() {
   // Check for errors
   if (!db) {
     if (error) {
-      qDebug() << "GPodLoader error:" << error->message;
+      qLog(Error) << "loading database failed:" << error->message;
       emit Error(QString::fromUtf8(error->message));
       g_error_free(error);
     } else {
@@ -65,20 +66,17 @@ void GPodLoader::LoadDatabase() {
   }
 
   // Convert all the tracks from libgpod structs into Song classes
+  const QString prefix = path_prefix_.isEmpty()
+      ? QDir::fromNativeSeparators(mount_point_) : path_prefix_;
+
   SongList songs;
   for (GList* tracks = db->tracks ; tracks != NULL ; tracks = tracks->next) {
     Itdb_Track* track = static_cast<Itdb_Track*>(tracks->data);
 
     Song song;
-    song.InitFromItdb(track);
+    song.InitFromItdb(track, prefix);
     song.set_directory_id(1);
 
-    QString filename = (path_prefix_.isEmpty() ? mount_point_ : path_prefix_) +
-                       song.filename();
-    if (path_prefix_.isEmpty())
-      filename = QDir::cleanPath(QDir::fromNativeSeparators(filename));
-    song.set_filename(filename);
-
     if (type_ != Song::Type_Unknown)
       song.set_filetype(type_);
     songs << song;
