@@ -22,14 +22,17 @@
 #ifndef AMAROK_ENGINEBASE_H
 #define AMAROK_ENGINEBASE_H
 
-#include <QUrl>
-#include <QObject>
-#include <QList>
-
+#include <stdint.h>
 #include <sys/types.h>
+
 #include <vector>
+
 #include <boost/noncopyable.hpp>
 
+#include <QList>
+#include <QObject>
+#include <QUrl>
+
 #include "engine_fwd.h"
 
 namespace Engine {
@@ -98,6 +101,7 @@ class Base : public QObject, boost::noncopyable {
 
   virtual void SetEqualizerEnabled(bool) {}
   virtual void SetEqualizerParameters(int preamp, const QList<int>& bandGains) {}
+  virtual void SetStereoBalance(float value) {}
 
  signals:
   // Emitted when crossfading is enabled and the track is crossfade_duration_
@@ -144,6 +148,8 @@ class Base : public QObject, boost::noncopyable {
   bool autocrossfade_enabled_;
   bool crossfade_same_album_;
   int next_background_stream_id_;
+  bool fadeout_pause_enabled_;
+  qint64 fadeout_pause_duration_nanosec_;
 
  private:
   bool about_to_end_emitted_;
