@@ -27,11 +27,12 @@ class LibrarySearchProvider : public BlockingSearchProvider {
 public:
   LibrarySearchProvider(LibraryBackendInterface* backend, const QString& name,
                         const QString& id, const QIcon& icon,
-                        bool enabled_by_default, QObject* parent = 0);
+                        bool enabled_by_default,
+                        Application* app, QObject* parent = 0);
 
   ResultList Search(int id, const QString& query);
-  void LoadTracksAsync(int id, const Result& result);
-  QString GetSuggestion();
+  MimeData* LoadTracks(const ResultList& results);
+  QStringList GetSuggestions(int count);
 
 private:
   LibraryBackendInterface* backend_;
