@@ -18,12 +18,12 @@
 #ifndef DIGITALLYIMPORTEDSERVICEBASE_H
 #define DIGITALLYIMPORTEDSERVICEBASE_H
 
+#include <boost/scoped_ptr.hpp>
+
 #include "digitallyimportedclient.h"
 #include "internetservice.h"
 #include "core/cachedlist.h"
 
-#include <boost/scoped_ptr.hpp>
-
 class DigitallyImportedClient;
 class DigitallyImportedUrlHandler;
 
@@ -73,7 +73,7 @@ signals:
   void StreamsChanged();
 
 private slots:
-  void LoadPlaylistFinished();
+  void LoadPlaylistFinished(QNetworkReply* reply);
   void Homepage();
   void ForceRefreshStreams();
   void RefreshStreams();
@@ -127,4 +127,9 @@ public:
   JazzRadioService(Application* app, InternetModel* model, QObject* parent = NULL);
 };
 
+class RockRadioService : public DigitallyImportedServiceBase {
+public:
+  RockRadioService(Application* app, InternetModel* model, QObject* parent = NULL);
+};
+
 #endif // DIGITALLYIMPORTEDSERVICEBASE_H
