@@ -43,6 +43,8 @@ QIcon DigitallyImportedUrlHandler::icon() const {
     return QIcon(":providers/skyfm.png");
   } else if (scheme() == "jazzradio") {
     return QIcon(":providers/jazzradio.png");
+  } else if (scheme() == "rockradio") {
+    return QIcon(":providers/rockradio.png");
   }
   return QIcon();
 }
