@@ -15,17 +15,24 @@
    along with Clementine.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "internetmodel.h"
+
+#include <QMimeData>
+#include <QtDebug>
+
 #include "digitallyimportedservicebase.h"
 #include "groovesharkservice.h"
 #include "icecastservice.h"
+#include "internetmimedata.h"
+#include "internetservice.h"
 #include "jamendoservice.h"
 #include "magnatuneservice.h"
-#include "internetmimedata.h"
-#include "internetmodel.h"
-#include "internetservice.h"
 #include "savedradio.h"
 #include "somafmservice.h"
 #include "soundcloudservice.h"
+#include "spotifyservice.h"
+#include "subsonicservice.h"
+#include "core/closure.h"
 #include "core/logging.h"
 #include "core/mergedproxymodel.h"
 #include "podcasts/podcastservice.h"
@@ -34,15 +41,21 @@
 #ifdef HAVE_LIBLASTFM
   #include "lastfmservice.h"
 #endif
-#ifdef HAVE_SPOTIFY
-  #include "spotifyservice.h"
-#endif
 #ifdef HAVE_GOOGLE_DRIVE
   #include "googledriveservice.h"
 #endif
-
-#include <QMimeData>
-#include <QtDebug>
+#ifdef HAVE_UBUNTU_ONE
+  #include "ubuntuoneservice.h"
+#endif
+#ifdef HAVE_DROPBOX
+  #include "dropboxservice.h"
+#endif
+#ifdef HAVE_SKYDRIVE
+  #include "skydriveservice.h"
+#endif
+#ifdef HAVE_BOX
+  #include "boxservice.h"
+#endif
 
 using smart_playlists::Generator;
 using smart_playlists::GeneratorMimeData;
@@ -75,12 +88,25 @@ InternetModel::InternetModel(Application* app, QObject* parent)
   AddService(new JazzRadioService(app, this));
   AddService(new MagnatuneService(app, this));
   AddService(new PodcastService(app, this));
+  AddService(new RadioGFMService(app, this));
+  AddService(new RockRadioService(app, this));
   AddService(new SavedRadio(app, this));
   AddService(new SkyFmService(app, this));
   AddService(new SomaFMService(app, this));
   AddService(new SoundCloudService(app, this));
-#ifdef HAVE_SPOTIFY
   AddService(new SpotifyService(app, this));
+  AddService(new SubsonicService(app, this));
+#ifdef HAVE_UBUNTU_ONE
+  AddService(new UbuntuOneService(app, this));
+#endif
+#ifdef HAVE_DROPBOX
+  AddService(new DropboxService(app, this));
+#endif
+#ifdef HAVE_SKYDRIVE
+  AddService(new SkydriveService(app, this));
+#endif
+#ifdef HAVE_BOX
+  AddService(new BoxService(app, this));
 #endif
 }
 
