@@ -94,6 +94,7 @@ MagnatuneService::MagnatuneService(Application* app, InternetModel* parent)
   library_sort_model_->setSourceModel(library_model_);
   library_sort_model_->setSortRole(LibraryModel::Role_SortText);
   library_sort_model_->setDynamicSortFilter(true);
+  library_sort_model_->setSortLocaleAware(true);
   library_sort_model_->sort(0);
 
   app_->player()->RegisterUrlHandler(url_handler_);
@@ -129,6 +130,9 @@ void MagnatuneService::LazyPopulate(QStandardItem* item) {
   switch (item->data(InternetModel::Role_Type).toInt()) {
     case InternetModel::Type_Service:
       library_model_->Init();
+      if (total_song_count_ == 0 && !load_database_task_id_) {
+        ReloadDatabase();
+      }
       model()->merged_model()->AddSubModel(item->index(), library_sort_model_);
       break;
 
@@ -139,9 +143,6 @@ void MagnatuneService::LazyPopulate(QStandardItem* item) {
 
 void MagnatuneService::UpdateTotalSongCount(int count) {
   total_song_count_ = count;
-  if (total_song_count_ == 0 && !load_database_task_id_) {
-    ReloadDatabase();
-  }
 }
 
 void MagnatuneService::ReloadDatabase() {
@@ -197,6 +198,7 @@ void MagnatuneService::ReloadDatabaseFinished() {
 
   // Add the songs to the database
   library_backend_->AddOrUpdateSongs(songs);
+  library_model_->Reset();
 }
 
 Song MagnatuneService::ReadTrack(QXmlStreamReader& reader) {
