@@ -27,13 +27,17 @@ class QSqlQuery;
 struct Directory {
   Directory() : id(-1) {}
 
+  bool operator ==(const Directory& other) const {
+    return path == other.path && id == other.id;
+  }
+
   QString path;
   int id;
 };
-Q_DECLARE_METATYPE(Directory);
+Q_DECLARE_METATYPE(Directory)
 
 typedef QList<Directory> DirectoryList;
-Q_DECLARE_METATYPE(DirectoryList);
+Q_DECLARE_METATYPE(DirectoryList)
 
 
 struct Subdirectory {
@@ -43,9 +47,9 @@ struct Subdirectory {
   QString path;
   uint mtime;
 };
-Q_DECLARE_METATYPE(Subdirectory);
+Q_DECLARE_METATYPE(Subdirectory)
 
 typedef QList<Subdirectory> SubdirectoryList;
-Q_DECLARE_METATYPE(SubdirectoryList);
+Q_DECLARE_METATYPE(SubdirectoryList)
 
 #endif // DIRECTORY_H
