@@ -74,7 +74,6 @@ public:
   virtual SongList GetSongs(
       const QString& artist, const QString& album, const QueryOptions& opt = QueryOptions()) = 0;
 
-  virtual bool HasCompilations(const QueryOptions& opt = QueryOptions()) = 0;
   virtual SongList GetCompilationSongs(const QString& album, const QueryOptions& opt = QueryOptions()) = 0;
 
   virtual AlbumList GetAllAlbums(const QueryOptions& opt = QueryOptions()) = 0;
@@ -104,6 +103,8 @@ class LibraryBackend : public LibraryBackendInterface {
   Q_OBJECT
 
  public:
+  static const char* kSettingsGroup;
+
   Q_INVOKABLE LibraryBackend(QObject* parent = 0);
   void Init(Database* db, const QString& songs_table,
             const QString& dirs_table, const QString& subdirs_table,
@@ -132,7 +133,6 @@ class LibraryBackend : public LibraryBackendInterface {
   SongList GetSongsByAlbum(const QString& album, const QueryOptions& opt = QueryOptions());
   SongList GetSongs(const QString& artist, const QString& album, const QueryOptions& opt = QueryOptions());
 
-  bool HasCompilations(const QueryOptions& opt = QueryOptions());
   SongList GetCompilationSongs(const QString& album, const QueryOptions& opt = QueryOptions());
 
   AlbumList GetAllAlbums(const QueryOptions& opt = QueryOptions());
@@ -157,6 +157,7 @@ class LibraryBackend : public LibraryBackendInterface {
   bool ExecQuery(LibraryQuery* q);
   SongList ExecLibraryQuery(LibraryQuery* query);
   SongList FindSongs(const smart_playlists::Search& search);
+  SongList GetAllSongs();
 
   void IncrementPlayCountAsync(int id);
   void IncrementSkipCountAsync(int id, float progress);
@@ -165,6 +166,8 @@ class LibraryBackend : public LibraryBackendInterface {
 
   void DeleteAll();
 
+  void ReloadSettingsAsync();
+
  public slots:
   void LoadDirectories();
   void UpdateTotalSongCount();
@@ -180,6 +183,7 @@ class LibraryBackend : public LibraryBackendInterface {
   void IncrementSkipCount(int id, float progress);
   void ResetStatistics(int id);
   void UpdateSongRating(int id, float rating);
+  void ReloadSettings();
 
  signals:
   void DirectoryDiscovered(const Directory& dir, const SubdirectoryList& subdirs);
@@ -188,6 +192,7 @@ class LibraryBackend : public LibraryBackendInterface {
   void SongsDiscovered(const SongList& songs);
   void SongsDeleted(const SongList& songs);
   void SongsStatisticsChanged(const SongList& songs);
+  void SongsRatingChanged(const SongList& songs);
   void DatabaseReset();
 
   void TotalSongCountUpdated(int total);
@@ -222,6 +227,8 @@ class LibraryBackend : public LibraryBackendInterface {
   QString dirs_table_;
   QString subdirs_table_;
   QString fts_table_;
+  bool save_statistics_in_file_;
+  bool save_ratings_in_file_;
 };
 
 #endif // LIBRARYBACKEND_H
