@@ -53,13 +53,17 @@ const char* LibraryModel::kSmartPlaylistsSettingsGroup = "SerialisedSmartPlaylis
 const int LibraryModel::kSmartPlaylistsVersion = 4;
 const int LibraryModel::kPrettyCoverSize = 32;
 
-typedef QFuture<SqlRowList> RootQueryFuture;
-typedef QFutureWatcher<SqlRowList> RootQueryWatcher;
+typedef QFuture<LibraryModel::QueryResult> RootQueryFuture;
+typedef QFutureWatcher<LibraryModel::QueryResult> RootQueryWatcher;
 
 static bool IsArtistGroupBy(const LibraryModel::GroupBy by) {
   return by == LibraryModel::GroupBy_Artist || by == LibraryModel::GroupBy_AlbumArtist;
 }
 
+static bool IsCompilationArtistNode(const LibraryItem* node) {
+  return node == node->parent->compilation_artist_node_;
+}
+
 LibraryModel::LibraryModel(LibraryBackend* backend, Application* app,
                            QObject* parent)
   : SimpleTreeModel<LibraryItem>(new LibraryItem(this), parent),
@@ -97,7 +101,8 @@ LibraryModel::LibraryModel(LibraryBackend* backend, Application* app,
 
   connect(backend_, SIGNAL(SongsDiscovered(SongList)), SLOT(SongsDiscovered(SongList)));
   connect(backend_, SIGNAL(SongsDeleted(SongList)), SLOT(SongsDeleted(SongList)));
-  connect(backend_, SIGNAL(SongsStatisticsChanged(SongList)), SLOT(SongsStatisticsChanged(SongList)));
+  connect(backend_, SIGNAL(SongsStatisticsChanged(SongList)), SLOT(SongsSlightlyChanged(SongList)));
+  connect(backend_, SIGNAL(SongsRatingChanged(SongList)), SLOT(SongsSlightlyChanged(SongList)));
   connect(backend_, SIGNAL(DatabaseReset()), SLOT(Reset()));
   connect(backend_, SIGNAL(TotalSongCountUpdated(int)), SLOT(TotalSongCountUpdatedSlot(int)));
 
@@ -162,12 +167,12 @@ void LibraryModel::SongsDiscovered(const SongList& songs) {
       GroupBy type = group_by_[i];
       if (type == GroupBy_None) break;
 
-      // Special case: if we're at the top level and the song is a compilation
-      // and the top level is Artists, then we want the Various Artists node :(
-      if (i == 0 && IsArtistGroupBy(type) && song.is_compilation()) {
-        if (compilation_artist_node_ == NULL)
-          CreateCompilationArtistNode(true, root_);
-        container = compilation_artist_node_;
+      // Special case: if the song is a compilation and the current GroupBy
+      // level is Artists, then we want the Various Artists node :(
+      if (IsArtistGroupBy(type) && song.is_compilation()) {
+        if (container->compilation_artist_node_ == NULL)
+          CreateCompilationArtistNode(true, container);
+        container = container->compilation_artist_node_;
       } else {
         // Otherwise find the proper container at this level based on the
         // item's key
@@ -176,6 +181,8 @@ void LibraryModel::SongsDiscovered(const SongList& songs) {
           case GroupBy_Album:       key = song.album(); break;
           case GroupBy_Artist:      key = song.artist(); break;
           case GroupBy_Composer:    key = song.composer(); break;
+          case GroupBy_Performer:   key = song.performer(); break;
+          case GroupBy_Grouping:    key = song.grouping(); break;
           case GroupBy_Genre:       key = song.genre(); break;
           case GroupBy_AlbumArtist: key = song.effective_albumartist(); break;
           case GroupBy_Year:
@@ -213,7 +220,7 @@ void LibraryModel::SongsDiscovered(const SongList& songs) {
   }
 }
 
-void LibraryModel::SongsStatisticsChanged(const SongList& songs) {
+void LibraryModel::SongsSlightlyChanged(const SongList& songs) {
   // This is called if there was a minor change to the songs that will not
   // normally require the library to be restructured.  We can just update our
   // internal cache of Song objects without worrying about resetting the model.
@@ -228,16 +235,17 @@ LibraryItem* LibraryModel::CreateCompilationArtistNode(bool signal, LibraryItem*
   if (signal)
     beginInsertRows(ItemToIndex(parent), parent->children.count(), parent->children.count());
 
-  compilation_artist_node_ =
+  parent->compilation_artist_node_ =
       new LibraryItem(LibraryItem::Type_Container, parent);
-  compilation_artist_node_->key = tr("Various artists");
-  compilation_artist_node_->sort_text = " various";
-  compilation_artist_node_->container_level = parent->container_level + 1;
+  parent->compilation_artist_node_->compilation_artist_node_ = NULL;
+  parent->compilation_artist_node_->key = tr("Various artists");
+  parent->compilation_artist_node_->sort_text = " various";
+  parent->compilation_artist_node_->container_level = parent->container_level + 1;
 
   if (signal)
     endInsertRows();
 
-  return compilation_artist_node_;
+  return parent->compilation_artist_node_;
 }
 
 QString LibraryModel::DividerKey(GroupBy type, LibraryItem* item) const {
@@ -251,6 +259,8 @@ QString LibraryModel::DividerKey(GroupBy type, LibraryItem* item) const {
   case GroupBy_Album:
   case GroupBy_Artist:
   case GroupBy_Composer:
+  case GroupBy_Performer:
+  case GroupBy_Grouping:
   case GroupBy_Genre:
   case GroupBy_AlbumArtist:
   case GroupBy_FileType: {
@@ -284,6 +294,8 @@ QString LibraryModel::DividerDisplayText(GroupBy type, const QString& key) const
   case GroupBy_Album:
   case GroupBy_Artist:
   case GroupBy_Composer:
+  case GroupBy_Performer:
+  case GroupBy_Grouping:
   case GroupBy_Genre:
   case GroupBy_AlbumArtist:
   case GroupBy_FileType:
@@ -352,8 +364,8 @@ void LibraryModel::SongsDeleted(const SongList& songs) {
         divider_keys << DividerKey(group_by_[0], node);
 
       // Special case the Various Artists node
-      if (node == compilation_artist_node_)
-        compilation_artist_node_ = NULL;
+      if (IsCompilationArtistNode(node))
+        node->parent->compilation_artist_node_ = NULL;
       else
         container_nodes_[node->container_level].remove(node->key);
 
@@ -554,44 +566,22 @@ QVariant LibraryModel::data(const LibraryItem* item, int role) const {
   return QVariant();
 }
 
-SqlRowList LibraryModel::RunRootQuery(const QueryOptions& query_options,
-                                      const Grouping& group_by) {
-  // Warning: Some copy-paste with LazyPopulate here
+bool LibraryModel::HasCompilations(const LibraryQuery& query) {
+  LibraryQuery q = query;
+  q.AddCompilationRequirement(true);
+  q.SetLimit(1);
 
-  // Information about what we want the children to be
-  GroupBy child_type = group_by[0];
-
-  // Initialise the query.  child_type says what type of thing we want (artists,
-  // songs, etc.)
-  LibraryQuery q(query_options);
-  InitQuery(child_type, &q);
-
-  // Top-level artists is special - we don't want compilation albums appearing
-  if (IsArtistGroupBy(child_type)) {
-    q.AddCompilationRequirement(false);
-  }
-
-  // Execute the query
   QMutexLocker l(backend_->db()->Mutex());
-  if (!backend_->ExecQuery(&q))
-    return SqlRowList();
+  if (!backend_->ExecQuery(&q)) return false;
 
-  SqlRowList rows;
-  while (q.Next()) {
-    rows << SqlRow(q);
-  }
-  return rows;
+  return q.Next();
 }
 
-void LibraryModel::LazyPopulate(LibraryItem* parent, bool signal) {
-  if (parent->lazy_loaded)
-    return;
-  parent->lazy_loaded = true;
-
-  // Warning: Some copy-paste with RunRootQuery here
+LibraryModel::QueryResult LibraryModel::RunQuery(LibraryItem* parent) {
+  QueryResult result;
 
   // Information about what we want the children to be
-  int child_level = parent->container_level + 1;
+  int child_level = parent == root_ ? 0 : parent->container_level + 1;
   GroupBy child_type = child_level >= 3 ? GroupBy_None : group_by_[child_level];
 
   // Initialise the query.  child_type says what type of thing we want (artists,
@@ -599,11 +589,6 @@ void LibraryModel::LazyPopulate(LibraryItem* parent, bool signal) {
   LibraryQuery q(query_options_);
   InitQuery(child_type, &q);
 
-  // Top-level artists is special - we don't want compilation albums appearing
-  if (child_level == 0 && IsArtistGroupBy(child_type)) {
-    q.AddCompilationRequirement(false);
-  }
-
   // Walk up through the item's parents adding filters as necessary
   LibraryItem* p = parent;
   while (p && p->type == LibraryItem::Type_Container) {
@@ -611,18 +596,44 @@ void LibraryModel::LazyPopulate(LibraryItem* parent, bool signal) {
     p = p->parent;
   }
 
+  // Artists GroupBy is special - we don't want compilation albums appearing
+  if (IsArtistGroupBy(child_type)) {
+    // Add the special Various artists node
+    if (show_various_artists_ && HasCompilations(q)) {
+      result.create_va = true;
+    }
+
+    // Don't show compilations again outside the Various artists node
+    q.AddCompilationRequirement(false);
+  }
+
   // Execute the query
   QMutexLocker l(backend_->db()->Mutex());
   if (!backend_->ExecQuery(&q))
-    return;
+    return result;
+
+  while (q.Next()) {
+    result.rows << SqlRow(q);
+  }
+  return result;
+}
+
+void LibraryModel::PostQuery(LibraryItem* parent,
+                             const LibraryModel::QueryResult& result,
+                             bool signal) {
+  // Information about what we want the children to be
+  int child_level = parent == root_ ? 0 : parent->container_level + 1;
+  GroupBy child_type = child_level >= 3 ? GroupBy_None : group_by_[child_level];
+
+  if (result.create_va) {
+    CreateCompilationArtistNode(signal, parent);
+  }
 
   // Step through the results
-  while (q.Next()) {
-    // Warning: Some copy-paste with ResetAsyncQueryFinished here
-
+  foreach (const SqlRow& row, result.rows) {
     // Create the item - it will get inserted into the model here
     LibraryItem* item =
-        ItemFromQuery(child_type, signal, child_level == 0, parent, SqlRow(q),
+        ItemFromQuery(child_type, signal, child_level == 0, parent, row,
                       child_level);
 
     // Save a pointer to it for later
@@ -633,9 +644,18 @@ void LibraryModel::LazyPopulate(LibraryItem* parent, bool signal) {
   }
 }
 
+void LibraryModel::LazyPopulate(LibraryItem* parent, bool signal) {
+  if (parent->lazy_loaded)
+    return;
+  parent->lazy_loaded = true;
+
+  QueryResult result = RunQuery(parent);
+  PostQuery(parent, result, signal);
+}
+
 void LibraryModel::ResetAsync() {
   RootQueryFuture future = QtConcurrent::run(
-        this, &LibraryModel::RunRootQuery, query_options_, group_by_);
+        this, &LibraryModel::RunQuery, root_);
   RootQueryWatcher* watcher = new RootQueryWatcher(this);
   watcher->setFuture(future);
 
@@ -644,27 +664,13 @@ void LibraryModel::ResetAsync() {
 
 void LibraryModel::ResetAsyncQueryFinished() {
   RootQueryWatcher* watcher = static_cast<RootQueryWatcher*>(sender());
-  const SqlRowList rows = watcher->result();
+  const struct QueryResult result = watcher->result();
   watcher->deleteLater();
 
   BeginReset();
   root_->lazy_loaded = true;
 
-  foreach (const SqlRow& row, rows) {
-    // Warning: Some copy-paste with LazyPopulate here
-
-    const GroupBy child_type = group_by_[0];
-
-    // Create the item - it will get inserted into the model here
-    LibraryItem* item =
-        ItemFromQuery(child_type, false, true, root_, row, 0);
-
-    // Save a pointer to it for later
-    if (child_type == GroupBy_None)
-      song_nodes_[item->metadata.id()] = item;
-    else
-      container_nodes_[0][item->key] = item;
-  }
+  PostQuery(root_, result, false);
 
   if (init_task_id_ != -1) {
     app_->task_manager()->SetTaskFinished(init_task_id_);
@@ -683,19 +689,12 @@ void LibraryModel::BeginReset() {
   container_nodes_[2].clear();
   divider_nodes_.clear();
   pending_art_.clear();
-  compilation_artist_node_ = NULL;
   smart_playlist_node_ = NULL;
 
   root_ = new LibraryItem(this);
+  root_->compilation_artist_node_ = NULL;
   root_->lazy_loaded = false;
 
-  if (show_various_artists_) {
-    // Various artists?
-    if (IsArtistGroupBy(group_by_[0]) &&
-        backend_->HasCompilations(query_options_))
-      CreateCompilationArtistNode(false, root_);
-  }
-
   // Smart playlists?
   if (show_smart_playlists_ && query_options_.filter().isEmpty())
     CreateSmartPlaylists();
@@ -722,6 +721,12 @@ void LibraryModel::InitQuery(GroupBy type, LibraryQuery* q) {
   case GroupBy_Composer:
     q->SetColumnSpec("DISTINCT composer");
     break;
+  case GroupBy_Performer:
+    q->SetColumnSpec("DISTINCT performer");
+    break;
+  case GroupBy_Grouping:
+    q->SetColumnSpec("DISTINCT grouping");
+    break;
   case GroupBy_YearAlbum:
     q->SetColumnSpec("DISTINCT year, album");
     break;
@@ -749,11 +754,11 @@ void LibraryModel::FilterQuery(GroupBy type, LibraryItem* item, LibraryQuery* q)
 
   switch (type) {
   case GroupBy_Artist:
-    if (item == compilation_artist_node_)
+    if (IsCompilationArtistNode(item))
       q->AddCompilationRequirement(true);
     else {
-      if (item->container_level == 0) // Stupid hack
-        q->AddCompilationRequirement(false);
+      // Don't duplicate compilations outside the Various artists node
+      q->AddCompilationRequirement(false);
       q->AddWhere("artist", item->key);
     }
     break;
@@ -770,15 +775,21 @@ void LibraryModel::FilterQuery(GroupBy type, LibraryItem* item, LibraryQuery* q)
   case GroupBy_Composer:
     q->AddWhere("composer", item->key);
     break;
+  case GroupBy_Performer:
+    q->AddWhere("performer", item->key);
+    break;
+  case GroupBy_Grouping:
+    q->AddWhere("grouping", item->key);
+    break;
   case GroupBy_Genre:
     q->AddWhere("genre", item->key);
     break;
   case GroupBy_AlbumArtist:
-    if (item == compilation_artist_node_)
+    if (IsCompilationArtistNode(item))
       q->AddCompilationRequirement(true);
     else {
-      if (item->container_level == 0) // Same stupid hack as above
-        q->AddCompilationRequirement(false);
+      // Don't duplicate compilations outside the Various artists node
+      q->AddCompilationRequirement(false);
       q->AddWhere("effective_albumartist", item->key);
     }
     break;
@@ -803,6 +814,7 @@ LibraryItem* LibraryModel::InitItem(GroupBy type, bool signal, LibraryItem *pare
 
   // Initialise the item depending on what type it's meant to be
   LibraryItem* item = new LibraryItem(item_type, parent);
+  item->compilation_artist_node_ = NULL;
   item->container_level = container_level;
   return item;
 }
@@ -836,6 +848,8 @@ LibraryItem* LibraryModel::ItemFromQuery(GroupBy type,
     break;
 
   case GroupBy_Composer:
+  case GroupBy_Performer:
+  case GroupBy_Grouping:
   case GroupBy_Genre:
   case GroupBy_Album:
   case GroupBy_AlbumArtist:
@@ -890,6 +904,8 @@ LibraryItem* LibraryModel::ItemFromSong(GroupBy type,
     break;
 
   case GroupBy_Composer:                      item->key = s.composer();
+  case GroupBy_Performer:                     item->key = s.performer();
+  case GroupBy_Grouping:                      item->key = s.grouping();
   case GroupBy_Genre: if (item->key.isNull()) item->key = s.genre();
   case GroupBy_Album: if (item->key.isNull()) item->key = s.album();
   case GroupBy_AlbumArtist: if (item->key.isNull()) item->key = s.effective_albumartist();
