@@ -80,6 +80,8 @@ class LibraryModel : public SimpleTreeModel<LibraryItem> {
     GroupBy_Genre = 6,
     GroupBy_AlbumArtist = 7,
     GroupBy_FileType = 8,
+    GroupBy_Performer = 9,
+    GroupBy_Grouping = 10,
   };
 
   struct Grouping {
@@ -104,6 +106,13 @@ class LibraryModel : public SimpleTreeModel<LibraryItem> {
     }
   };
 
+  struct QueryResult {
+    QueryResult() : create_va(false) {}
+
+    SqlRowList rows;
+    bool create_va;
+  };
+
   LibraryBackend* backend() const { return backend_; }
   LibraryDirectoryModel* directory_model() const { return dir_model_; }
 
@@ -174,7 +183,7 @@ class LibraryModel : public SimpleTreeModel<LibraryItem> {
   // From LibraryBackend
   void SongsDiscovered(const SongList& songs);
   void SongsDeleted(const SongList& songs);
-  void SongsStatisticsChanged(const SongList& songs);
+  void SongsSlightlyChanged(const SongList& songs);
   void TotalSongCountUpdatedSlot(int count);
 
   // Called after ResetAsync
@@ -186,8 +195,10 @@ class LibraryModel : public SimpleTreeModel<LibraryItem> {
   // Provides some optimisations for loading the list of items in the root.
   // This gets called a lot when filtering the playlist, so it's nice to be
   // able to do it in a background thread.
-  SqlRowList RunRootQuery(const QueryOptions& query_options,
-                          const Grouping& group_by);
+  QueryResult RunQuery(LibraryItem* parent);
+  void PostQuery(LibraryItem* parent, const QueryResult& result, bool signal);
+
+  bool HasCompilations(const LibraryQuery& query);
 
   void BeginReset();
 
@@ -253,9 +264,6 @@ class LibraryModel : public SimpleTreeModel<LibraryItem> {
   // Keyed on a letter, a year, a century, etc.
   QMap<QString, LibraryItem*> divider_nodes_;
 
-  // Only applies if the first level is "artist"
-  LibraryItem* compilation_artist_node_;
-
   // Only applies if smart playlists are set to on
   LibraryItem* smart_playlist_node_;
 
