@@ -34,7 +34,7 @@ class LibraryPlaylistItem : public PlaylistItem {
 
   QUrl Url() const;
 
-  bool IsLocalLibraryItem() const { return song_.id() != -1; }
+  bool IsLocalLibraryItem() const { return true; }
 
  protected:
   QVariant DatabaseValue(DatabaseColumn column) const;
