@@ -26,11 +26,9 @@
 
 #include <boost/scoped_ptr.hpp>
 
-class DeviceManager;
+class Application;
 class LibraryFilterWidget;
-class LibraryModel;
 class OrganiseDialog;
-class TaskManager;
 
 class QMimeData;
 
@@ -62,10 +60,7 @@ class LibraryView : public AutoExpandingTreeView {
   // this will return all of it's songs.
   SongList GetSelectedSongs() const;
 
-  void SetCoverProviders(CoverProviders* cover_providers);
-  void SetTaskManager(TaskManager* task_manager);
-  void SetLibrary(LibraryModel* library);
-  void SetDeviceManager(DeviceManager* device_manager);
+  void SetApplication(Application* app);
   void SetFilter(LibraryFilterWidget* filter);
 
   // QTreeView
@@ -78,6 +73,9 @@ class LibraryView : public AutoExpandingTreeView {
 
   void FilterReturnPressed();
 
+  void SaveFocus();
+  void RestoreFocus();
+
  signals:
   void ShowConfigDialog();
 
@@ -112,12 +110,11 @@ class LibraryView : public AutoExpandingTreeView {
  private:
   void RecheckIsEmpty();
   void ShowInVarious(bool on);
+  bool RestoreLevelFocus(const QModelIndex& parent = QModelIndex());
+  void SaveContainerPath(const QModelIndex& child);
 
  private:
-  CoverProviders* cover_providers_;
-  LibraryModel* library_;
-  DeviceManager* devices_;
-  TaskManager* task_manager_;
+  Application* app_;
   LibraryFilterWidget* filter_;
 
   int total_song_count_;
@@ -147,6 +144,11 @@ class LibraryView : public AutoExpandingTreeView {
   boost::scoped_ptr<EditTagDialog> edit_tag_dialog_;
 
   bool is_in_keyboard_search_;
+
+  // Save focus
+  Song last_selected_song_;
+  QString last_selected_container_;
+  QSet<QString> last_selected_path_;
 };
 
 #endif // LIBRARYVIEW_H
