@@ -16,18 +16,19 @@
 */
 
 #include "musicbrainzclient.h"
-#include "core/logging.h"
-#include "core/network.h"
-#include "core/utilities.h"
 
 #include <QCoreApplication>
 #include <QNetworkReply>
 #include <QSet>
 #include <QXmlStreamReader>
-#include <QtDebug>
+
+#include "core/closure.h"
+#include "core/logging.h"
+#include "core/network.h"
+#include "core/utilities.h"
 
 const char* MusicBrainzClient::kTrackUrl = "http://musicbrainz.org/ws/2/recording/";
-const char* MusicBrainzClient::kDiscUrl = "http://musicbrainz.org/ws/1/release/";
+const char* MusicBrainzClient::kDiscUrl = "http://musicbrainz.org/ws/2/discid/";
 const char* MusicBrainzClient::kDateRegex = "^[12]\\d{3}";
 const int MusicBrainzClient::kDefaultTimeout = 5000; // msec
 
@@ -49,8 +50,9 @@ void MusicBrainzClient::Start(int id, const QString& mbid) {
   QNetworkRequest req(url);
 
   QNetworkReply* reply = network_->get(req);
-  connect(reply, SIGNAL(finished()), SLOT(RequestFinished()));
-  requests_[reply] = id;
+  NewClosure(reply, SIGNAL(finished()), this,
+             SLOT(RequestFinished(QNetworkReply*, int)), reply, id);
+  requests_[id] = reply;
 
   timeouts_->AddReply(reply);
 }
@@ -59,35 +61,29 @@ void MusicBrainzClient::StartDiscIdRequest(const QString& discid) {
   typedef QPair<QString, QString> Param;
 
   QList<Param> parameters;
-  parameters << Param("type", "xml")
-             << Param("discid", discid);
+  parameters << Param("inc", "artists+recordings");
 
-  QUrl url(kDiscUrl);
+  QUrl url(kDiscUrl + discid);
   url.setQueryItems(parameters);
   QNetworkRequest req(url);
 
   QNetworkReply* reply = network_->get(req);
-  connect(reply, SIGNAL(finished()), SLOT(DiscIdRequestFinished()));
-  //requests_[reply] = id;
+  NewClosure(reply, SIGNAL(finished()), this,
+             SLOT(DiscIdRequestFinished(QNetworkReply*)), reply);
 
   timeouts_->AddReply(reply);
 }
 
 void MusicBrainzClient::Cancel(int id) {
-  QNetworkReply* reply = requests_.key(id);
-  requests_.remove(reply);
-  delete reply;
+  delete requests_.take(id);
 }
 
 void MusicBrainzClient::CancelAll() {
-  qDeleteAll(requests_.keys());
+  qDeleteAll(requests_.values());
   requests_.clear();
 }
 
-void MusicBrainzClient::DiscIdRequestFinished() {
-  QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
-  if (!reply)
-    return;
+void MusicBrainzClient::DiscIdRequestFinished(QNetworkReply* reply) {
   reply->deleteLater();
 
   ResultList ret;
@@ -120,7 +116,7 @@ void MusicBrainzClient::DiscIdRequestFinished() {
 
   while (!reader.atEnd()) {
     QXmlStreamReader::TokenType token = reader.readNext();
-    if (token == QXmlStreamReader::StartElement && reader.name() == "track") {
+    if (token == QXmlStreamReader::StartElement && reader.name() == "recording") {
       ResultList tracks = ParseTrack(&reader);
       foreach (const Result& track, tracks) {
         if (!track.title_.isEmpty()) {
@@ -136,16 +132,9 @@ void MusicBrainzClient::DiscIdRequestFinished() {
 }
 
 
-void MusicBrainzClient::RequestFinished() {
-  QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
-  if (!reply)
-    return;
-
+void MusicBrainzClient::RequestFinished(QNetworkReply* reply, int id) {
   reply->deleteLater();
-  if (!requests_.contains(reply))
-    return;
-
-  int id = requests_.take(reply);
+  requests_.remove(id);
   ResultList ret;
 
   if (reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt() != 200) {
@@ -195,8 +184,12 @@ MusicBrainzClient::ResultList MusicBrainzClient::ParseTrack(QXmlStreamReader* re
   }
 
   ResultList ret;
-  foreach (const Release& release, releases) {
-    ret << release.CopyAndMergeInto(result);
+  if (releases.isEmpty()) {
+    ret << result;
+  } else {
+    foreach (const Release& release, releases) {
+      ret << release.CopyAndMergeInto(result);
+    }
   }
   return ret;
 }
