@@ -64,6 +64,14 @@
 #include <algorithm>
 #include <boost/bind.hpp>
 
+#ifdef USE_STD_UNORDERED_MAP
+  #include <unordered_map>
+  using std::unordered_map;
+#else
+  #include <tr1/unordered_map>
+  using std::tr1::unordered_map;
+#endif
+
 using smart_playlists::Generator;
 using smart_playlists::GeneratorInserter;
 using smart_playlists::GeneratorPtr;
@@ -90,6 +98,7 @@ Playlist::Playlist(PlaylistBackend* backend,
                    LibraryBackend* library,
                    int id,
                    const QString& special_type,
+                   bool favorite,
                    QObject *parent)
   : QAbstractListModel(parent),
     is_loading_(false),
@@ -99,6 +108,7 @@ Playlist::Playlist(PlaylistBackend* backend,
     task_manager_(task_manager),
     library_(library),
     id_(id),
+    favorite_(favorite),
     current_is_paused_(false),
     current_virtual_index_(-1),
     is_shuffled_(false),
@@ -166,6 +176,8 @@ bool Playlist::column_is_editable(Playlist::Column column) {
     case Column_Album:
     case Column_AlbumArtist:
     case Column_Composer:
+    case Column_Performer:
+    case Column_Grouping:
     case Column_Track:
     case Column_Disc:
     case Column_Year:
@@ -201,6 +213,12 @@ bool Playlist::set_column_value(Song& song, Playlist::Column column,
     case Column_Composer:
       song.set_composer(value.toString());
       break;
+    case Column_Performer:
+      song.set_performer(value.toString());
+      break;
+    case Column_Grouping:
+      song.set_grouping(value.toString());
+      break;
     case Column_Track:
       song.set_track(value.toInt());
       break;
@@ -263,6 +281,8 @@ QVariant Playlist::data(const QModelIndex& index, int role) const {
         case Column_Genre:        return song.genre();
         case Column_AlbumArtist:  return song.playlist_albumartist();
         case Column_Composer:     return song.composer();
+        case Column_Performer:    return song.performer();
+        case Column_Grouping:     return song.grouping();
 
         case Column_Rating:       return song.rating();
         case Column_PlayCount:    return song.playcount();
@@ -1152,6 +1172,8 @@ bool Playlist::CompareItems(int column, Qt::SortOrder order,
     case Column_Genre:        strcmp(genre);
     case Column_AlbumArtist:  strcmp(playlist_albumartist);
     case Column_Composer:     strcmp(composer);
+    case Column_Performer:    strcmp(performer);
+    case Column_Grouping:     strcmp(grouping);
 
     case Column_Rating:       cmp(rating);
     case Column_PlayCount:    cmp(playcount);
@@ -1191,6 +1213,8 @@ QString Playlist::column_name(Column column) {
     case Column_Genre:        return tr("Genre");
     case Column_AlbumArtist:  return tr("Album artist");
     case Column_Composer:     return tr("Composer");
+    case Column_Performer:    return tr("Performer");
+    case Column_Grouping:     return tr("Grouping");
 
     case Column_Rating:       return tr("Rating");
     case Column_PlayCount:    return tr("Play count");
@@ -1919,18 +1943,44 @@ void Playlist::RemoveDeletedSongs() {
   removeRows(rows_to_remove);
 }
 
+struct SongSimilarHash {
+  long operator() (const Song& song) const {
+    return HashSimilar(song);
+  }
+};
+
+struct SongSimilarEqual {
+  long operator() (const Song& song1, const Song& song2) const {
+    return song1.IsSimilar(song2);
+  }
+};
+
 void Playlist::RemoveDuplicateSongs() {
   QList<int> rows_to_remove;
-  QSet<QUrl> filenames;
+  unordered_map<Song, int, SongSimilarHash, SongSimilarEqual> unique_songs;
 
   for (int row = 0; row < items_.count(); ++row) {
     PlaylistItemPtr item = items_[row];
-    Song song = item->Metadata();
+    const Song& song = item->Metadata();
 
-    if (filenames.contains(song.url())) {
-      rows_to_remove.append(row);
-    } else {
-      filenames.insert(song.url());
+    bool found_duplicate = false;
+
+    auto uniq_song_it = unique_songs.find(song);
+    if (uniq_song_it != unique_songs.end()) {
+      const Song& uniq_song = uniq_song_it->first;
+
+      if (song.bitrate() > uniq_song.bitrate()) {
+        rows_to_remove.append(unique_songs[uniq_song]);
+        unique_songs.erase(uniq_song);
+        unique_songs.insert(std::make_pair(song, row));
+      } else {
+        rows_to_remove.append(row);
+      }
+      found_duplicate = true;
+    }
+
+    if (!found_duplicate) {
+      unique_songs.insert(std::make_pair(song, row));
     }
   }
 
@@ -1960,7 +2010,7 @@ bool Playlist::ApplyValidityOnCurrentSong(const QUrl& url, bool valid) {
     }
   }
 
-  return current;
+  return static_cast<bool>(current);
 }
 
 void Playlist::SetColumnAlignment(const ColumnAlignmentMap& alignment) {
