@@ -34,7 +34,7 @@ class PlaylistItem : public boost::enable_shared_from_this<PlaylistItem> {
  public:
   PlaylistItem(const QString& type)
     : type_(type) {}
-  virtual ~PlaylistItem() {}
+  virtual ~PlaylistItem();
 
   static PlaylistItem* NewFromType(const QString& type);
   static PlaylistItem* NewFromSongsTable(const QString& table, const Song& song);
