@@ -34,6 +34,7 @@ public:
   PlaylistTabBar(QWidget *parent = 0);
 
   static const int kDragHoverTimeout = 500;
+  static const char* kSettingsGroup;
 
   void SetActions(QAction* new_playlist, QAction* load_playlist);
   void SetManager(PlaylistManager* manager);
@@ -50,15 +51,15 @@ public:
   void set_text_by_id(int id, const QString& text);
 
   void RemoveTab(int id);
-  void InsertTab(int id, int index, const QString& text,
-                 const QIcon& icon = QIcon());
+  void InsertTab(int id, int index, const QString& text, bool favorite);
 
 signals:
   void CurrentIdChanged(int id);
   void Rename(int id, const QString& name);
-  void Remove(int id);
+  void Close(int id);
   void Save(int id);
   void PlaylistOrderChanged(const QList<int>& ids);
+  void PlaylistFavorited(int id, bool favorite);
 
 protected:
   void contextMenuEvent(QContextMenuEvent* e);
@@ -76,8 +77,11 @@ private slots:
   void Rename();
   void RenameInline();
   void HideEditor();
-  void Remove();
-  void RemoveFromTabIndex(int index);
+  void Close();
+  void CloseFromTabIndex(int index);
+  // Used when playlist's favorite flag isn't changed from the favorite widget
+  // (e.g. from the playlistlistcontainer): will update the favorite widget
+  void PlaylistFavoritedSlot(int id, bool favorite);
   void TabMoved();
   void Save();
 
@@ -88,7 +92,7 @@ private:
   int menu_index_;
   QAction* new_;
   QAction* rename_;
-  QAction* remove_;
+  QAction* close_;
   QAction* save_;
 
   QBasicTimer drag_hover_timer_;
@@ -98,9 +102,6 @@ private:
 
   // Editor for inline renaming
   RenameTabLineEdit* rename_editor_;
-
-  // We want to ask for confirmation only in certain cases
-  bool removing_with_confirm_;
 };
 
 #endif // PLAYLISTTABBAR_H
