@@ -20,17 +20,19 @@
 
 #include <QMimeData>
 
+#include "core/mimedata.h"
 #include "core/song.h"
 
-class LibraryBackend;
+class LibraryBackendInterface;
 
-class SongMimeData : public QMimeData {
+class SongMimeData : public MimeData {
   Q_OBJECT
 
- public:
-  SongMimeData() : backend(NULL) {}
+public:
+  SongMimeData()
+    : backend(NULL) {}
 
-  LibraryBackend* backend;
+  LibraryBackendInterface* backend;
   SongList songs;
 };
 
