@@ -36,7 +36,7 @@ class M3UParser : public ParserBase {
 
   bool TryMagic(const QByteArray &data) const;
 
-  SongList Load(QIODevice* device, const QDir& dir = QDir()) const;
+  SongList Load(QIODevice* device, const QString& playlist_path = "", const QDir& dir = QDir()) const;
   void Save(const SongList &songs, QIODevice* device, const QDir& dir = QDir()) const;
 
  private:
@@ -50,7 +50,7 @@ class M3UParser : public ParserBase {
     Metadata() : length(-1) {}
     QString artist;
     QString title;
-    int length;
+    qint64 length;
   };
 
   bool ParseMetadata(const QString& line, Metadata* metadata) const;
