@@ -18,6 +18,7 @@
 #ifndef PLAYLISTPARSER_H
 #define PLAYLISTPARSER_H
 
+#include <QDir>
 #include <QObject>
 
 #include "core/song.h"
@@ -39,11 +40,13 @@ public:
   QString default_extension() const;
   QString default_filter() const;
 
-  ParserBase* MaybeGetParserForMagic(const QByteArray& data,
-                                     const QString& mime_type = QString()) const;
+  ParserBase* ParserForMagic(const QByteArray& data,
+                             const QString& mime_type = QString()) const;
   ParserBase* ParserForExtension(const QString& suffix) const;
 
-  SongList Load(const QString& filename, ParserBase* parser = 0) const;
+  SongList LoadFromFile(const QString& filename) const;
+  SongList LoadFromDevice(QIODevice* device, const QString& path_hint = QString(),
+                          const QDir& dir_hint = QDir()) const;
   void Save(const SongList& songs, const QString& filename) const;
 
 private:
