@@ -17,15 +17,16 @@
 
 #include "generator.h"
 #include "querygenerator.h"
-#include "radio/jamendodynamicplaylist.h"
+#include "core/logging.h"
+#include "internet/jamendodynamicplaylist.h"
 
 #include <QSettings>
 
 namespace smart_playlists {
 
 const int Generator::kDefaultLimit = 20;
-const int Generator::kDynamicHistory = 5;
-const int Generator::kDynamicFuture = 15;
+const int Generator::kDefaultDynamicHistory = 5;
+const int Generator::kDefaultDynamicFuture = 15;
 
 Generator::Generator()
   : QObject(NULL),
@@ -39,7 +40,7 @@ GeneratorPtr Generator::Create(const QString& type) {
   else if (type == "Jamendo")
     return GeneratorPtr(new JamendoDynamicPlaylist);
 
-  qWarning() << "Invalid playlist generator type:" << type;
+  qLog(Warning) << "Invalid playlist generator type:" << type;
   return GeneratorPtr();
 }
 
