@@ -21,14 +21,16 @@
 #include <QMimeData>
 
 #include "generator_fwd.h"
+#include "core/mimedata.h"
 
 namespace smart_playlists {
 
-class GeneratorMimeData : public QMimeData {
+class GeneratorMimeData : public MimeData {
   Q_OBJECT
 
 public:
-  GeneratorMimeData(GeneratorPtr generator) : generator_(generator) {}
+  GeneratorMimeData(GeneratorPtr generator)
+    : generator_(generator) {}
 
   GeneratorPtr generator_;
 };
