@@ -56,7 +56,7 @@ void SearchPreview::set_application(Application* app) {
 void SearchPreview::set_library(LibraryBackend* backend) {
   backend_ = backend;
 
-  model_ = new Playlist(NULL, NULL, backend_, -1, QString(), this);
+  model_ = new Playlist(NULL, NULL, backend_, -1, QString(), false, this);
   ui_->tree->setModel(model_);
   ui_->tree->SetPlaylist(model_);
   ui_->tree->SetItemDelegates(backend_);
