@@ -23,12 +23,15 @@
 #include <QWheelEvent>
 #include <QtDebug>
 
+#include "core/logging.h"
+
 const qreal SongInfoTextView::kDefaultFontSize = 8.5;
 const char* SongInfoTextView::kSettingsGroup = "SongInfo";
 
 SongInfoTextView::SongInfoTextView(QWidget* parent)
   : QTextBrowser(parent),
-    last_width_(-1)
+    last_width_(-1),
+    recursion_filter_(false)
 {
   setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
   setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
@@ -37,14 +40,20 @@ SongInfoTextView::SongInfoTextView(QWidget* parent)
   ReloadSettings();
 }
 
-void SongInfoTextView::ReloadSettings() {
+qreal SongInfoTextView::FontSize() {
   QSettings s;
   s.beginGroup(kSettingsGroup);
+  return s.value("font_size", kDefaultFontSize).toReal();
+}
 
-  qreal size = s.value("font_size", kDefaultFontSize).toReal();
+QFont SongInfoTextView::Font() {
   QFont font;
-  font.setPointSizeF(size);
-  document()->setDefaultFont(font);
+  font.setPointSizeF(FontSize());
+  return font;
+}
+
+void SongInfoTextView::ReloadSettings() {
+  document()->setDefaultFont(Font());
 }
 
 void SongInfoTextView::resizeEvent(QResizeEvent* e) {
@@ -88,3 +97,19 @@ void SongInfoTextView::SetHtml(const QString& html) {
 
   setHtml(copy);
 }
+
+// Prevents QTextDocument from trying to load remote images before they are
+// ready.
+QVariant SongInfoTextView::loadResource(int type, const QUrl& name) {
+  if (recursion_filter_) {
+    recursion_filter_ = false;
+    return QVariant();
+  }
+  recursion_filter_ = true;
+  if (type == QTextDocument::ImageResource && name.scheme() == "http") {
+    if (document()->resource(type, name).isNull()) {
+      return QVariant();
+    }
+  }
+  return QTextBrowser::loadResource(type, name);
+}
