@@ -49,7 +49,8 @@ About::About(QWidget *parent)
              << Person("Paul Cifarelli", "paul@cifarelli.net")
              << Person("Felipe Rivera", "liebremx@users.sourceforge.net")
              << Person("Alexander Peitz")
-             << Person("Artur Rona", "artur.rona@gmail.com");
+             << Person("Artur Rona", "artur.rona@gmail.com")
+             << Person("Andreas Muttscheller", "asfa194@gmail.com");
 
   qSort(authors_);
   qSort(thanks_to_);
@@ -79,11 +80,9 @@ QString About::MakeHtml() const {
   ret += QString("<br /><a href=\"http://www.smitelli.com/?page=blog&p=54\">Scott Smitelli</a>");
   ret += QString("<br /><a href=\"http://hyperboleandahalf.blogspot.com\">Allie Brosh</a></p>");
 
-#ifdef HAVE_SPOTIFY
-  ret += "<p>This product uses SPOTIFY(R) CORE but is not endorsed, certified "
-         "or otherwise approved in any way by Spotify. Spotify is the "
-         "registered trade mark of the Spotify Group.</p>";
-#endif // HAVE_SPOTIFY
+  ret += "<p>This product uses Music by Spotify but is not endorsed, certified "
+         "or otherwise approved in any way by Spotify. Spotify is the registered "
+         "trade mark of the Spotify Group.</p>";
 
   return ret;
 }
