@@ -22,6 +22,7 @@
 #include "core/application.h"
 #include "core/logging.h"
 #include "core/utilities.h"
+#include "covers/albumcoverexporter.h"
 #include "covers/albumcoverfetcher.h"
 #include "covers/albumcoverloader.h"
 #include "covers/coverproviders.h"
@@ -32,8 +33,10 @@
 #include "playlist/songmimedata.h"
 #include "widgets/forcescrollperpixel.h"
 #include "ui/albumcoverchoicecontroller.h"
+#include "ui/albumcoverexport.h"
 
 #include <QActionGroup>
+#include <QPushButton>
 #include <QContextMenuEvent>
 #include <QEvent>
 #include <QFileDialog>
@@ -51,6 +54,7 @@
 const char* AlbumCoverManager::kSettingsGroup = "CoverManager";
 
 AlbumCoverManager::AlbumCoverManager(Application* app,
+                                     LibraryBackend* library_backend,
                                      QWidget* parent,
                                      QNetworkAccessManager* network)
   : QMainWindow(parent),
@@ -59,17 +63,22 @@ AlbumCoverManager::AlbumCoverManager(Application* app,
     album_cover_choice_controller_(new AlbumCoverChoiceController(this)),
     cover_fetcher_(new AlbumCoverFetcher(app_->cover_providers(), this, network)),
     cover_searcher_(NULL),
+    cover_export_(NULL),
+    cover_exporter_(new AlbumCoverExporter(this)),
     artist_icon_(IconLoader::Load("x-clementine-artist")),
     all_artists_icon_(IconLoader::Load("x-clementine-album")),
     context_menu_(new QMenu(this)),
     progress_bar_(new QProgressBar(this)),
-    jobs_(0)
+    abort_progress_(new QPushButton(this)),
+    jobs_(0),
+    library_backend_(library_backend)
 {
   ui_->setupUi(this);
   ui_->albums->set_cover_manager(this);
 
   // Icons
   ui_->action_fetch->setIcon(IconLoader::Load("download"));
+  ui_->export_covers->setIcon(IconLoader::Load("document-save"));
   ui_->view->setIcon(IconLoader::Load("view-choose"));
   ui_->fetch->setIcon(IconLoader::Load("download"));
   ui_->action_add_to_playlist->setIcon(IconLoader::Load("media-playback-start"));
@@ -89,10 +98,15 @@ AlbumCoverManager::AlbumCoverManager(Application* app,
   no_cover_icon_ = QPixmap::fromImage(square_nocover);
 
   cover_searcher_ = new AlbumCoverSearcher(no_cover_icon_, app_, this);
+  cover_export_ = new AlbumCoverExport(this);
 
   // Set up the status bar
   statusBar()->addPermanentWidget(progress_bar_);
+  statusBar()->addPermanentWidget(abort_progress_);
   progress_bar_->hide();
+  abort_progress_->hide();
+  abort_progress_->setText(tr("Abort"));
+  connect(abort_progress_, SIGNAL(clicked()), this, SLOT(CancelRequests()));
 
   ui_->albums->setAttribute(Qt::WA_MacShowFocusRect, false);
   ui_->artists->setAttribute(Qt::WA_MacShowFocusRect, false);
@@ -100,7 +114,7 @@ AlbumCoverManager::AlbumCoverManager(Application* app,
   QShortcut* close = new QShortcut(QKeySequence::Close, this);
   connect(close, SIGNAL(activated()), SLOT(close()));
 
-  ResetFetchCoversButton();
+  EnableCoversButtons();
 }
 
 AlbumCoverManager::~AlbumCoverManager() {
@@ -110,7 +124,7 @@ AlbumCoverManager::~AlbumCoverManager() {
 }
 
 LibraryBackend* AlbumCoverManager::backend() const {
-  return app_->library_backend();
+  return library_backend_;
 }
 
 void AlbumCoverManager::Init() {
@@ -147,6 +161,9 @@ void AlbumCoverManager::Init() {
   connect(album_cover_choice_controller_->show_cover_action(),
           SIGNAL(triggered()), this, SLOT(ShowCover()));
 
+  connect(cover_exporter_, SIGNAL(AlbumCoversExportUpdate(int, int, int)),
+          SLOT(UpdateExportStatus(int, int, int)));
+
   context_menu_->addActions(actions);
   context_menu_->addSeparator();
   context_menu_->addAction(ui_->action_load);
@@ -161,6 +178,7 @@ void AlbumCoverManager::Init() {
   connect(filter_group, SIGNAL(triggered(QAction*)), SLOT(UpdateFilter()));
   connect(ui_->view, SIGNAL(clicked()), ui_->view, SLOT(showMenu()));
   connect(ui_->fetch, SIGNAL(clicked()), SLOT(FetchAlbumCovers()));
+  connect(ui_->export_covers, SIGNAL(clicked()), SLOT(ExportCovers()));
   connect(cover_fetcher_, SIGNAL(AlbumCoverFetched(quint64,QImage,CoverSearchStatistics)),
           SLOT(AlbumCoverFetched(quint64,QImage,CoverSearchStatistics)));
   connect(ui_->action_fetch, SIGNAL(triggered()), SLOT(FetchSingleCover()));
@@ -221,11 +239,14 @@ void AlbumCoverManager::CancelRequests() {
         QSet<quint64>::fromList(cover_loading_tasks_.keys()));
   cover_loading_tasks_.clear();
 
+  cover_exporter_->Cancel();
+
   cover_fetching_tasks_.clear();
   cover_fetcher_->Clear();
   progress_bar_->hide();
+  abort_progress_->hide();
   statusBar()->clearMessage();
-  ResetFetchCoversButton();
+  EnableCoversButtons();
 }
 
 static bool CompareNocase(const QString& left, const QString& right) {
@@ -238,13 +259,13 @@ static bool CompareAlbumNameNocase(const LibraryBackend::Album& left,
 }
 
 void AlbumCoverManager::Reset() {
-  ResetFetchCoversButton();
+  EnableCoversButtons();
 
   ui_->artists->clear();
   new QListWidgetItem(all_artists_icon_, tr("All artists"), ui_->artists, All_Artists);
   new QListWidgetItem(artist_icon_, tr("Various artists"), ui_->artists, Various_Artists);
 
-  QStringList artists(app_->library_backend()->GetAllArtistsWithAlbums());
+  QStringList artists(library_backend_->GetAllArtistsWithAlbums());
   qStableSort(artists.begin(), artists.end(), CompareNocase);
 
   foreach (const QString& artist, artists) {
@@ -255,8 +276,14 @@ void AlbumCoverManager::Reset() {
   }
 }
 
-void AlbumCoverManager::ResetFetchCoversButton() {
+void AlbumCoverManager::EnableCoversButtons() {
   ui_->fetch->setEnabled(app_->cover_providers()->HasAnyProviders());
+  ui_->export_covers->setEnabled(true);
+}
+
+void AlbumCoverManager::DisableCoversButtons() {
+  ui_->fetch->setEnabled(false);
+  ui_->export_covers->setEnabled(false);
 }
 
 void AlbumCoverManager::ArtistChanged(QListWidgetItem* current) {
@@ -275,10 +302,10 @@ void AlbumCoverManager::ArtistChanged(QListWidgetItem* current) {
   // selected in the artist list.
   LibraryBackend::AlbumList albums;
   switch (current->type()) {
-    case Various_Artists: albums = app_->library_backend()->GetCompilationAlbums(); break;
-    case Specific_Artist: albums = app_->library_backend()->GetAlbumsByArtist(current->text()); break;
+    case Various_Artists: albums = library_backend_->GetCompilationAlbums(); break;
+    case Specific_Artist: albums = library_backend_->GetAlbumsByArtist(current->text()); break;
     case All_Artists:
-    default:              albums = app_->library_backend()->GetAllAlbums(); break;
+    default:              albums = library_backend_->GetAllAlbums(); break;
   }
 
   // Sort by album name.  The list is already sorted by sqlite but it was done
@@ -336,10 +363,24 @@ void AlbumCoverManager::UpdateFilter() {
     hide = Hide_WithoutCovers;
   }
 
+  qint32 total_count = 0;
+  qint32 without_cover = 0;
+
   for (int i = 0; i < ui_->albums->count(); ++i) {
     QListWidgetItem* item = ui_->albums->item(i);
-    item->setHidden(ShouldHide(*item, filter, hide));
+    bool should_hide = ShouldHide(*item, filter, hide);
+    item->setHidden(should_hide);
+
+    if (!should_hide) {
+      total_count++;
+      if (item->icon().cacheKey() == no_cover_icon_.cacheKey()) {
+        without_cover++;
+      }
+    }
   }
+
+  ui_->total_albums->setText(QString::number(total_count));
+  ui_->without_cover->setText(QString::number(without_cover));
 }
 
 bool AlbumCoverManager::ShouldHide(
@@ -385,6 +426,7 @@ void AlbumCoverManager::FetchAlbumCovers() {
 
   progress_bar_->setMaximum(jobs_);
   progress_bar_->show();
+  abort_progress_->show();
   fetch_statistics_ = CoverSearchStatistics();
   UpdateStatusText();
 }
@@ -400,7 +442,7 @@ void AlbumCoverManager::AlbumCoverFetched(quint64 id, const QImage& image,
   }
 
   if (cover_fetching_tasks_.isEmpty()) {
-    ResetFetchCoversButton();
+    EnableCoversButtons();
   }
 
   fetch_statistics_ += statistics;
@@ -425,6 +467,7 @@ void AlbumCoverManager::UpdateStatusText() {
   if (cover_fetching_tasks_.isEmpty()) {
     QTimer::singleShot(2000, statusBar(), SLOT(clearMessage()));
     progress_bar_->hide();
+    abort_progress_->hide();
 
     CoverSearchStatisticsDialog* dialog = new CoverSearchStatisticsDialog(this);
     dialog->setAttribute(Qt::WA_DeleteOnClose);
@@ -514,6 +557,7 @@ void AlbumCoverManager::FetchSingleCover() {
 
   progress_bar_->setMaximum(jobs_);
   progress_bar_->show();
+  abort_progress_->show();
   UpdateStatusText();
 }
 
@@ -634,7 +678,7 @@ SongList AlbumCoverManager::GetSongsInAlbum(const QModelIndex& index) const {
   if (!artist.isEmpty())
     q.AddWhere("artist", artist);
 
-  if (!app_->library_backend()->ExecQuery(&q))
+  if (!library_backend_->ExecQuery(&q))
     return ret;
 
   while (q.Next()) {
@@ -659,7 +703,7 @@ SongMimeData* AlbumCoverManager::GetMimeDataForAlbums(const QModelIndexList& ind
     return NULL;
 
   SongMimeData* data = new SongMimeData;
-  data->backend = app_->library_backend();
+  data->backend = library_backend_;
   data->songs = songs;
   return data;
 }
@@ -691,7 +735,7 @@ void AlbumCoverManager::SaveAndSetCover(QListWidgetItem *item, const QImage &ima
   QString path = album_cover_choice_controller_->SaveCoverInCache(artist, album, image);
 
   // Save the image in the database
-  app_->library_backend()->UpdateManualAlbumArtAsync(artist, album, path);
+  library_backend_->UpdateManualAlbumArtAsync(artist, album, path);
 
   // Update the icon in our list
   quint64 id = app_->album_cover_loader()->LoadImageAsync(
@@ -699,3 +743,63 @@ void AlbumCoverManager::SaveAndSetCover(QListWidgetItem *item, const QImage &ima
   item->setData(Role_PathManual, path);
   cover_loading_tasks_[id] = item;
 }
+
+void AlbumCoverManager::ExportCovers() {
+  AlbumCoverExport::DialogResult result = cover_export_->Exec();
+
+  if(result.cancelled_) {
+    return;
+  }
+
+  DisableCoversButtons();
+
+  cover_exporter_->SetDialogResult(result);
+
+  for (int i=0 ; i<ui_->albums->count() ; ++i) {
+    QListWidgetItem* item = ui_->albums->item(i);
+
+    // skip hidden and coverless albums
+    if (item->isHidden() || item->icon().cacheKey() == no_cover_icon_.cacheKey()) {
+      continue;
+    }
+
+    cover_exporter_->AddExportRequest(ItemAsSong(item));
+  }
+
+  if (cover_exporter_->request_count() > 0) {
+    progress_bar_->setMaximum(cover_exporter_->request_count());
+    progress_bar_->show();
+    abort_progress_->show();
+
+    cover_exporter_->StartExporting();
+  } else {
+    QMessageBox msg;
+    msg.setWindowTitle(tr("Export finished"));
+    msg.setText(tr("No covers to export."));
+    msg.exec();
+  }
+}
+
+void AlbumCoverManager::UpdateExportStatus(int exported, int skipped, int max) {
+  progress_bar_->setValue(exported);
+
+  QString message = tr("Exported %1 covers out of %2 (%3 skipped)")
+                      .arg(exported)
+                      .arg(max)
+                      .arg(skipped);
+  statusBar()->showMessage(message);
+
+  // end of the current process
+  if(exported + skipped >= max) {
+    QTimer::singleShot(2000, statusBar(), SLOT(clearMessage()));
+
+    progress_bar_->hide();
+    abort_progress_->hide();
+    EnableCoversButtons();
+
+    QMessageBox msg;
+    msg.setWindowTitle(tr("Export finished"));
+    msg.setText(message);
+    msg.exec();
+  }
+}
