@@ -17,6 +17,7 @@
 
 #include "albumcovermanager.h"
 #include "albumcovermanagerlist.h"
+#include "library/librarybackend.h"
 #include "playlist/songmimedata.h"
 
 #include <boost/scoped_ptr.hpp>
@@ -43,7 +44,7 @@ QMimeData* AlbumCoverManagerList::mimeData(const QList<QListWidgetItem*> items)
   // Get URLs from the songs
   QList<QUrl> urls;
   foreach (const Song& song, songs) {
-    urls << QUrl::fromLocalFile(song.filename());
+    urls << song.url();
   }
 
   // Get the QAbstractItemModel data so the picture works
