@@ -17,6 +17,7 @@
 
 #include "macsystemtrayicon.h"
 
+#include "core/mac_delegate.h"
 #include "core/song.h"
 
 #include <QAction>
@@ -83,7 +84,7 @@ class MacSystemTrayIconPrivate : boost::noncopyable {
 
     // Don't look now.
     // This must be called after our custom NSApplicationDelegate has been set.
-    [[NSApp delegate] setDockMenu:dock_menu_];
+    [(AppDelegate*)([NSApp delegate]) setDockMenu:dock_menu_];
 
     ClearNowPlaying();
   }
@@ -202,7 +203,5 @@ void MacSystemTrayIcon::ClearNowPlaying() {
 }
 
 void MacSystemTrayIcon::SetNowPlaying(const Song& song, const QString& image_path) {
-  // TODO: implement a rich tooltip with cover art, similar to the one used in QT environment
-  // see SetNowPlaying's documentation in systemtrayicon.h
   p_->ShowNowPlaying(song.artist(), song.PrettyTitle());
 }
