@@ -23,6 +23,7 @@
 #include "core/commandlineoptions.h"
 #include "core/database.h"
 #include "core/deletefiles.h"
+#include "core/filesystemmusicstorage.h"
 #include "core/globalshortcuts.h"
 #include "core/logging.h"
 #include "core/mac_startup.h"
@@ -35,6 +36,7 @@
 #include "core/songloader.h"
 #include "core/stylesheetloader.h"
 #include "core/taskmanager.h"
+#include "core/timeconstants.h"
 #include "core/utilities.h"
 #include "devices/devicemanager.h"
 #include "devices/devicestatefiltermodel.h"
@@ -58,6 +60,7 @@
 #include "musicbrainz/tagfetcher.h"
 #include "playlist/playlistbackend.h"
 #include "playlist/playlist.h"
+#include "playlist/playlistlistcontainer.h"
 #include "playlist/playlistmanager.h"
 #include "playlist/playlistsequence.h"
 #include "playlist/playlistview.h"
@@ -74,6 +77,7 @@
 #include "ui/about.h"
 #include "ui/addstreamdialog.h"
 #include "ui/albumcovermanager.h"
+#include "ui/console.h"
 #include "ui/edittagdialog.h"
 #include "ui/equalizer.h"
 #include "ui/iconloader.h"
@@ -165,6 +169,7 @@ MainWindow::MainWindow(Application* app,
     global_search_view_(new GlobalSearchView(app_, this)),
     library_view_(new LibraryViewContainer(this)),
     file_view_(new FileView(this)),
+    playlist_list_(new PlaylistListContainer(this)),
     internet_view_(new InternetViewContainer(this)),
     device_view_(new DeviceView(this)),
     song_info_view_(new SongInfoView(this)),
@@ -187,6 +192,8 @@ MainWindow::MainWindow(Application* app,
     library_sort_model_(new QSortFilterProxyModel(this)),
     track_position_timer_(new QTimer(this)),
     was_maximized_(false),
+    saved_playback_position_(0),
+    saved_playback_state_(Engine::Empty),
     doubleclick_addmode_(AddBehaviour_Append),
     doubleclick_playmode_(PlayBehaviour_IfStopped),
     menu_playmode_(PlayBehaviour_IfStopped)
@@ -227,6 +234,7 @@ MainWindow::MainWindow(Application* app,
   ui_->tabs->AddTab(global_search_view_, IconLoader::Load("search"), tr("Search"));
   ui_->tabs->AddTab(library_view_, IconLoader::Load("folder-sound"), tr("Library"));
   ui_->tabs->AddTab(file_view_, IconLoader::Load("document-open"), tr("Files"));
+  ui_->tabs->AddTab(playlist_list_, IconLoader::Load("view-media-playlist"), tr("Playlists"));
   ui_->tabs->AddTab(internet_view_, IconLoader::Load("applications-internet"), tr("Internet"));
   ui_->tabs->AddTab(device_view_, IconLoader::Load("multimedia-player-ipod-mini-blue"), tr("Devices"));
   ui_->tabs->AddSpacer();
@@ -252,6 +260,7 @@ MainWindow::MainWindow(Application* app,
   library_sort_model_->setSourceModel(app_->library()->model());
   library_sort_model_->setSortRole(LibraryModel::Role_SortText);
   library_sort_model_->setDynamicSortFilter(true);
+  library_sort_model_->setSortLocaleAware(true);
   library_sort_model_->sort(0);
 
   connect(ui_->playlist, SIGNAL(ViewSelectionModelChanged()), SLOT(PlaylistViewSelectionModelChanged()));
@@ -262,6 +271,7 @@ MainWindow::MainWindow(Application* app,
   library_view_->view()->SetApplication(app_);
   internet_view_->SetApplication(app_);
   device_view_->SetApplication(app_);
+  playlist_list_->SetApplication(app_);
 
   organise_dialog_->SetDestinationModel(app_->library()->model()->directory_model());
 
@@ -343,6 +353,7 @@ MainWindow::MainWindow(Application* app,
   connect(ui_->action_update_library, SIGNAL(triggered()), app_->library(), SLOT(IncrementalScan()));
   connect(ui_->action_full_library_scan, SIGNAL(triggered()), app_->library(), SLOT(FullScan()));
   connect(ui_->action_queue_manager, SIGNAL(triggered()), SLOT(ShowQueueManager()));
+  connect(ui_->action_add_files_to_transcoder, SIGNAL(triggered()), SLOT(AddFilesToTranscoder()));
 
   background_streams_->AddAction("Rain", ui_->action_rain);
   background_streams_->AddAction("Hypnotoad", ui_->action_hypnotoad);
@@ -368,8 +379,9 @@ MainWindow::MainWindow(Application* app,
   ui_->ban_button->setDefaultAction(ui_->action_ban);
   ui_->scrobbling_button->setDefaultAction(ui_->action_toggle_scrobbling);
   ui_->clear_playlist_button->setDefaultAction(ui_->action_clear_playlist);
-  ui_->playlist->SetActions(ui_->action_new_playlist, ui_->action_save_playlist,
+  ui_->playlist->SetActions(ui_->action_new_playlist,
                             ui_->action_load_playlist,
+                            ui_->action_save_playlist,
                             ui_->action_next_playlist,    /* These two actions aren't associated */
                             ui_->action_previous_playlist /* to a button but to the main window */ );
 
@@ -486,6 +498,7 @@ MainWindow::MainWindow(Application* app,
   playlist_menu_->addAction(ui_->action_renumber_tracks);
   playlist_menu_->addAction(ui_->action_selection_set_value);
   playlist_menu_->addAction(ui_->action_auto_complete_tags);
+  playlist_menu_->addAction(ui_->action_add_files_to_transcoder);
   playlist_menu_->addSeparator();
   playlist_copy_to_library_ = playlist_menu_->addAction(IconLoader::Load("edit-copy"), tr("Copy to library..."), this, SLOT(PlaylistCopyToLibrary()));
   playlist_move_to_library_ = playlist_menu_->addAction(IconLoader::Load("go-jump"), tr("Move to library..."), this, SLOT(PlaylistMoveToLibrary()));
@@ -621,6 +634,7 @@ MainWindow::MainWindow(Application* app,
   // Analyzer
   ui_->analyzer->SetEngine(app_->player()->engine());
   ui_->analyzer->SetActions(ui_->action_visualisations);
+  connect(ui_->analyzer, SIGNAL(WheelEvent(int)), SLOT(VolumeWheelEvent(int)));
 
   // Equalizer
   qLog(Debug) << "Creating equalizer";
@@ -628,9 +642,12 @@ MainWindow::MainWindow(Application* app,
           app_->player()->engine(), SLOT(SetEqualizerParameters(int,QList<int>)));
   connect(equalizer_.get(), SIGNAL(EnabledChanged(bool)),
           app_->player()->engine(), SLOT(SetEqualizerEnabled(bool)));
+  connect(equalizer_.get(), SIGNAL(StereoBalanceChanged(float)),
+          app_->player()->engine(), SLOT(SetStereoBalance(float)));
   app_->player()->engine()->SetEqualizerEnabled(equalizer_->is_enabled());
   app_->player()->engine()->SetEqualizerParameters(
       equalizer_->preamp_value(), equalizer_->gain_values());
+  app_->player()->engine()->SetStereoBalance(equalizer_->stereo_balance());
 
   // Statusbar widgets
   ui_->playlist_summary->setMinimumWidth(QFontMetrics(font()).width("WW selected of WW tracks - [ WW:WW ]"));
@@ -653,6 +670,8 @@ MainWindow::MainWindow(Application* app,
           SLOT(NowPlayingWidgetPositionChanged(bool)));
   connect(ui_->action_hypnotoad, SIGNAL(toggled(bool)), ui_->now_playing, SLOT(AllHail(bool)));
   connect(ui_->action_kittens, SIGNAL(toggled(bool)), ui_->now_playing, SLOT(EnableKittens(bool)));
+  // Hide the console
+  //connect(ui_->action_console, SIGNAL(triggered()), SLOT(ShowConsole()));
   NowPlayingWidgetPositionChanged(ui_->now_playing->show_above_status_bar());
 
   // Load theme
@@ -741,6 +760,8 @@ MainWindow::MainWindow(Application* app,
 
   CheckFullRescanRevisions();
 
+  LoadPlaybackStatus();
+
   qLog(Debug) << "Started";
 }
 
@@ -865,6 +886,7 @@ void MainWindow::VolumeChanged(int volume) {
 
 void MainWindow::SongChanged(const Song& song) {
   setWindowTitle(song.PrettyTitleWithArtist());
+  tray_icon_->SetProgress(0);
 
 #ifdef HAVE_LIBLASTFM
   if (ui_->action_toggle_scrobbling->isVisible())
@@ -883,7 +905,13 @@ void MainWindow::TrackSkipped(PlaylistItemPtr item) {
     const qint64 length = app_->player()->engine()->length_nanosec();
     const float percentage = (length == 0 ? 1 : float(position) / length);
 
-    app_->library_backend()->IncrementSkipCountAsync(song.id(), percentage);
+    const qint64 seconds_left = (length - position) / kNsecPerSec;
+    const qint64 seconds_total = length / kNsecPerSec;
+
+    if (((0.05 * seconds_total > 60 && percentage < 0.98) || percentage < 0.95) &&
+        seconds_left > 5) { // Never count the skip if under 5 seconds left
+      app_->library_backend()->IncrementSkipCountAsync(song.id(), percentage);
+    }
   }
 }
 
@@ -944,6 +972,45 @@ void MainWindow::SaveGeometry() {
   settings_.setValue("tab_mode", ui_->tabs->mode());
 }
 
+void MainWindow::SavePlaybackStatus() {
+  QSettings settings;
+  settings.beginGroup(MainWindow::kSettingsGroup);
+  settings.setValue("playback_state", app_->player()->GetState());
+  if (app_->player()->GetState() == Engine::Playing ||
+          app_->player()->GetState() == Engine::Paused) {
+    settings.setValue("playback_position", app_->player()->engine()->position_nanosec() / kNsecPerSec);
+  } else {
+    settings.setValue("playback_position", 0);
+  }
+}
+
+void MainWindow::LoadPlaybackStatus() {
+  QSettings settings;
+  settings.beginGroup(MainWindow::kSettingsGroup);
+  bool resume_playback = settings.value("resume_playback_after_start", false).toBool();
+  saved_playback_state_ = static_cast<Engine::State>
+          (settings.value("playback_state", Engine::Empty).toInt());
+  saved_playback_position_ = settings.value("playback_position", 0).toDouble();
+  if (!resume_playback ||
+        saved_playback_state_ == Engine::Empty ||
+        saved_playback_state_ == Engine::Idle) {
+    return;
+  }
+
+  QTimer::singleShot(100, this, SLOT(ResumePlayback()));
+}
+
+void MainWindow::ResumePlayback() {
+  qLog(Debug) << "Resuming playback";
+  app_->player()->Play();
+
+  app_->player()->SeekTo(saved_playback_position_);
+
+  if (saved_playback_state_ == Engine::Paused) {
+    app_->player()->Pause();
+  }
+}
+
 void MainWindow::PlayIndex(const QModelIndex& index) {
   if (!index.isValid())
     return;
@@ -1362,7 +1429,7 @@ void MainWindow::PlaylistRightClick(const QPoint& global_pos, const QModelIndex&
   add_to_another_menu->setIcon(IconLoader::Load("list-add"));
 
   PlaylistBackend::Playlist playlist;
-  foreach (playlist, app_->playlist_backend()->GetAllPlaylists()) {
+  foreach (playlist, app_->playlist_backend()->GetAllOpenPlaylists()) {
     //don't add the current playlist
     if (playlist.id != app_->playlist_manager()->current()->id()) {
       QAction* existing_playlist = new QAction(this);
@@ -1646,6 +1713,9 @@ void MainWindow::CommandlineOptionsReceived(const CommandlineOptions &options) {
     case CommandlineOptions::Player_Next:
       app_->player()->Next();
       break;
+    case CommandlineOptions::Player_RestartOrPrevious:
+      app_->player()->RestartOrPrevious();
+      break;
 
     case CommandlineOptions::Player_None:
       break;
@@ -1718,6 +1788,28 @@ void MainWindow::PlaylistUndoRedoChanged(QAction *undo, QAction *redo) {
   playlist_menu_->insertAction(playlist_undoredo_, redo);
 }
 
+void MainWindow::AddFilesToTranscoder() {
+  if (!transcode_dialog_) {
+    transcode_dialog_.reset(new TranscodeDialog);
+  }
+
+  QStringList filenames;
+
+  foreach (const QModelIndex& index,
+           ui_->playlist->view()->selectionModel()->selection().indexes()) {
+    if (index.column() != 0)
+      continue;
+    int row = app_->playlist_manager()->current()->proxy()->mapToSource(index).row();
+    PlaylistItemPtr item(app_->playlist_manager()->current()->item_at(row));
+    Song song = item->Metadata();
+    filenames << song.url().toLocalFile();
+  }
+
+  transcode_dialog_->SetFilenames(filenames);
+
+  ShowTranscodeDialog();
+}
+
 void MainWindow::ShowLibraryConfig() {
   EnsureSettingsDialogCreated();
   settings_dialog_->OpenAtPage(SettingsDialog::Page_Library);
@@ -1810,17 +1902,12 @@ void MainWindow::PlaylistOrganiseSelected(bool copy) {
 void MainWindow::PlaylistDelete() {
   // Note: copied from LibraryView::Delete
 
-  if (QMessageBox::question(this, tr("Delete files"),
-        tr("These files will be deleted from disk, are you sure you want to continue?"),
+  if (QMessageBox::warning(this, tr("Delete files"),
+        tr("These files will be permanently deleted from disk, are you sure you want to continue?"),
         QMessageBox::Yes, QMessageBox::Cancel) != QMessageBox::Yes)
     return;
 
-  // We can cheat and always take the storage of the first directory, since
-  // they'll all be FilesystemMusicStorage in a library and deleting doesn't
-  // check the actual directory.
-  boost::shared_ptr<MusicStorage> storage =
-      app_->library_model()->directory_model()->index(0, 0).data(MusicStorage::Role_Storage)
-      .value<boost::shared_ptr<MusicStorage> >();
+  boost::shared_ptr<MusicStorage> storage(new FilesystemMusicStorage("/"));
 
   // Get selected songs
   SongList selected_songs;
@@ -1843,7 +1930,7 @@ void MainWindow::PlaylistOpenInBrowser() {
 
   foreach (const QModelIndex& proxy_index, proxy_indexes) {
     const QModelIndex index = app_->playlist_manager()->current()->proxy()->mapToSource(proxy_index);
-    urls << index.sibling(index.row(), Playlist::Column_Filename).data().toString();
+    urls << QUrl(index.sibling(index.row(), Playlist::Column_Filename).data().toString());
   }
 
   Utilities::OpenInFileBrowser(urls);
@@ -1900,7 +1987,7 @@ void MainWindow::ChangeLibraryQueryMode(QAction* action) {
 
 void MainWindow::ShowCoverManager() {
   if (!cover_manager_) {
-    cover_manager_.reset(new AlbumCoverManager(app_));
+    cover_manager_.reset(new AlbumCoverManager(app_, app_->library_backend()));
     cover_manager_->Init();
 
     // Cover manager connections
@@ -2084,6 +2171,7 @@ bool MainWindow::winEvent(MSG* msg, long*) {
 #endif // Q_OS_WIN32
 
 void MainWindow::Exit() {
+  SavePlaybackStatus();
   if(app_->player()->engine()->is_fadeout_enabled()) {
     // To shut down the application when fadeout will be finished
     connect(app_->player()->engine(), SIGNAL(FadeoutFinishedSignal()), qApp, SLOT(quit()));
@@ -2231,6 +2319,7 @@ void MainWindow::HandleNotificationPreview(OSD::Behaviour type, QString line1, Q
     fake.Init("Title", "Artist", "Album", 123);
     fake.set_genre("Classical");
     fake.set_composer("Anonymous");
+    fake.set_performer("Anonymous");
     fake.set_track(1);
     fake.set_disc(1);
     fake.set_year(2011);
@@ -2257,3 +2346,23 @@ void MainWindow::DoGlobalSearch(const QString& query) {
   FocusGlobalSearchField();
   global_search_view_->StartSearch(query);
 }
+
+void MainWindow::ShowConsole() {
+  Console* console = new Console(app_, this);
+  console->show();
+}
+
+void MainWindow::keyPressEvent(QKeyEvent* event) {
+  if(event->key() == Qt::Key_Space) {
+    app_->player()->PlayPause();
+    event->accept();
+  } else if(event->key() == Qt::Key_Left) {
+    ui_->track_slider->Seek(-1);
+    event->accept();
+  } else if(event->key() == Qt::Key_Right) {
+    ui_->track_slider->Seek(1);
+    event->accept();
+  } else {
+    QMainWindow::keyPressEvent(event);
+  }
+}
