@@ -60,6 +60,7 @@ class OrganiseDialog;
 class OSD;
 class Player;
 class PlaylistBackend;
+class PlaylistListContainer;
 class PlaylistManager;
 class QueueManager;
 class InternetItem;
@@ -120,6 +121,7 @@ class MainWindow : public QMainWindow, public PlatformInterface {
   void CommandlineOptionsReceived(const CommandlineOptions& options);
 
  protected:
+  void keyPressEvent(QKeyEvent* event);
   void resizeEvent(QResizeEvent* event);
   void closeEvent(QCloseEvent* event);
 
@@ -156,6 +158,7 @@ class MainWindow : public QMainWindow, public PlatformInterface {
   void AutoCompleteTags();
   void AutoCompleteTagsAccepted();
   void PlaylistUndoRedoChanged(QAction* undo, QAction* redo);
+  void AddFilesToTranscoder();
 
   void PlaylistCopyToLibrary();
   void PlaylistMoveToLibrary();
@@ -237,6 +240,9 @@ class MainWindow : public QMainWindow, public PlatformInterface {
   void ShowSongInfoConfig();
 
   void SaveGeometry();
+  void SavePlaybackStatus();
+  void LoadPlaybackStatus();
+  void ResumePlayback();
 
   void AddSongInfoGenerator(smart_playlists::GeneratorPtr gen);
 
@@ -252,6 +258,8 @@ class MainWindow : public QMainWindow, public PlatformInterface {
   void FocusGlobalSearchField();
   void DoGlobalSearch(const QString& query);
 
+  void ShowConsole();
+
  private:
   void ConnectInfoView(SongInfoBase* view);
 
@@ -279,6 +287,7 @@ class MainWindow : public QMainWindow, public PlatformInterface {
   GlobalSearchView* global_search_view_;
   LibraryViewContainer* library_view_;
   FileView* file_view_;
+  PlaylistListContainer* playlist_list_;
   InternetViewContainer* internet_view_;
   DeviceView* device_view_;
   SongInfoView* song_info_view_;
@@ -331,6 +340,8 @@ class MainWindow : public QMainWindow, public PlatformInterface {
   QSettings settings_;
 
   bool was_maximized_;
+  int saved_playback_position_;
+  Engine::State saved_playback_state_;
   AddBehaviour doubleclick_addmode_;
   PlayBehaviour doubleclick_playmode_;
   PlayBehaviour menu_playmode_;
