@@ -18,6 +18,8 @@
 #include "organiseerrordialog.h"
 #include "ui_organiseerrordialog.h"
 
+#include <QUrl>
+
 OrganiseErrorDialog::OrganiseErrorDialog(QWidget *parent)
   : QDialog(parent),
     ui_(new Ui_OrganiseErrorDialog)
@@ -38,7 +40,7 @@ void OrganiseErrorDialog::Show(
     OperationType type, const SongList& songs_with_errors) {
   QStringList files;
   foreach (const Song& song, songs_with_errors) {
-    files << song.filename();
+    files << song.url().toLocalFile();
   }
   Show(type, files);
 }
