@@ -35,6 +35,7 @@ QtSystemTrayIcon::QtSystemTrayIcon(QObject* parent)
     action_play_pause_(NULL),
     action_stop_(NULL),
     action_stop_after_this_track_(NULL),
+    action_mute_(NULL),
     action_love_(NULL),
     action_ban_(NULL)
 {
@@ -77,7 +78,21 @@ bool QtSystemTrayIcon::eventFilter(QObject* object, QEvent* event) {
 
   if (event->type() == QEvent::Wheel) {
     QWheelEvent* e = static_cast<QWheelEvent*>(event);
-    emit ChangeVolume(e->delta());
+    if (e->modifiers() == Qt::ShiftModifier) {
+      if (e->delta() > 0) {
+        emit SeekForward();
+      } else {
+        emit SeekBackward();
+      }
+    } else if (e->modifiers() == Qt::ControlModifier) {
+      if (e->delta() < 0) {
+        emit NextTrack();
+      } else {
+        emit PreviousTrack();
+      }
+    } else {
+      emit ChangeVolume(e->delta());
+    }
     return true;
   }
 
@@ -97,14 +112,18 @@ void QtSystemTrayIcon::SetupMenu(
   menu_->addAction(next->icon(), next->text(), next, SLOT(trigger()));
 
   menu_->addSeparator();
-  menu_->addAction(mute->icon(), mute->text(), mute, SLOT(trigger()));
+  action_mute_ = menu_->addAction(mute->icon(), mute->text(), mute, SLOT(trigger()));
+  action_mute_->setCheckable(true);
+  action_mute_->setChecked(mute->isChecked());
 
   menu_->addSeparator();
 #ifdef HAVE_LIBLASTFM
   action_love_ = menu_->addAction(love->icon(), love->text(), love, SLOT(trigger()));
   action_love_->setVisible(love->isVisible());
+  action_love_->setEnabled(love->isEnabled());
   action_ban_ = menu_->addAction(ban->icon(), ban->text(), ban, SLOT(trigger()));
   action_ban_->setVisible(ban->isVisible());
+  action_ban_->setEnabled(ban->isEnabled());
 #endif
 
   menu_->addSeparator();
@@ -180,6 +199,30 @@ void QtSystemTrayIcon::SetStopped() {
 #endif
 }
 
+void QtSystemTrayIcon::LastFMButtonVisibilityChanged(bool value) {
+#ifdef HAVE_LIBLASTFM
+  action_ban_->setVisible(value);
+  action_love_->setVisible(value);
+#endif
+}
+
+void QtSystemTrayIcon::LastFMButtonLoveStateChanged(bool value) {
+#ifdef HAVE_LIBLASTFM
+  action_love_->setEnabled(value);
+#endif
+}
+
+void QtSystemTrayIcon::LastFMButtonBanStateChanged(bool value) {
+#ifdef HAVE_LIBLASTFM
+  action_ban_->setEnabled(value);
+#endif
+}
+
+void QtSystemTrayIcon::MuteButtonStateChanged(bool value) {
+  if (action_mute_)
+    action_mute_->setChecked(value);
+}
+
 bool QtSystemTrayIcon::IsVisible() const {
   return tray_->isVisible();
 }
