@@ -23,6 +23,7 @@
 #include "iconloader.h"
 #include "playbacksettingspage.h"
 #include "networkproxysettingspage.h"
+#include "networkremotesettingspage.h"
 #include "notificationssettingspage.h"
 #include "mainwindow.h"
 #include "settingsdialog.h"
@@ -37,6 +38,9 @@
 #include "internet/digitallyimportedsettingspage.h"
 #include "internet/groovesharksettingspage.h"
 #include "internet/magnatunesettingspage.h"
+#include "internet/spotifysettingspage.h"
+#include "internet/subsonicsettingspage.h"
+#include "internet/ubuntuonesettingspage.h"
 #include "library/librarysettingspage.h"
 #include "playlist/playlistview.h"
 #include "podcasts/podcastsettingspage.h"
@@ -55,14 +59,24 @@
 # include "wiimotedev/wiimotesettingspage.h"
 #endif
 
-#ifdef HAVE_SPOTIFY
-# include "internet/spotifysettingspage.h"
-#endif
 
 #ifdef HAVE_GOOGLE_DRIVE
 # include "internet/googledrivesettingspage.h"
 #endif
 
+#ifdef HAVE_UBUNTU_ONE
+#  include "internet/ubuntuonesettingspage.h"
+#endif
+
+#ifdef HAVE_DROPBOX
+#  include "internet/dropboxsettingspage.h"
+#endif
+
+#ifdef HAVE_BOX
+#  include "internet/boxsettingspage.h"
+#endif
+
+#include <QAbstractButton>
 #include <QDesktopWidget>
 #include <QPainter>
 #include <QPushButton>
@@ -121,6 +135,7 @@ SettingsDialog::SettingsDialog(Application* app, BackgroundStreams* streams, QWi
   AddPage(Page_Library, new LibrarySettingsPage(this), general);
   AddPage(Page_Proxy, new NetworkProxySettingsPage(this), general);
   AddPage(Page_Transcoding, new TranscoderSettingsPage(this), general);
+  AddPage(Page_NetworkRemote, new NetworkRemoteSettingsPage(this), general);
 
 #ifdef HAVE_WIIMOTEDEV
   AddPage(Page_Wiimotedev, new WiimoteSettingsPage(this), general);
@@ -147,13 +162,23 @@ SettingsDialog::SettingsDialog(Application* app, BackgroundStreams* streams, QWi
   AddPage(Page_GoogleDrive, new GoogleDriveSettingsPage(this), providers);
 #endif
 
-#ifdef HAVE_SPOTIFY
-  AddPage(Page_Spotify, new SpotifySettingsPage(this), providers);
+#ifdef HAVE_UBUNTU_ONE
+  AddPage(Page_UbuntuOne, new UbuntuOneSettingsPage(this), providers);
 #endif
 
+#ifdef HAVE_DROPBOX
+  AddPage(Page_Dropbox, new DropboxSettingsPage(this), providers);
+#endif
+
+#ifdef HAVE_BOX
+  AddPage(Page_Box, new BoxSettingsPage(this), providers);
+#endif
+
+  AddPage(Page_Spotify, new SpotifySettingsPage(this), providers);
   AddPage(Page_Magnatune, new MagnatuneSettingsPage(this), providers);
   AddPage(Page_DigitallyImported, new DigitallyImportedSettingsPage(this), providers);
   AddPage(Page_BackgroundStreams, new BackgroundStreamsSettingsPage(this), providers);
+  AddPage(Page_Subsonic, new SubsonicSettingsPage(this), providers);
   AddPage(Page_Podcasts, new PodcastSettingsPage(this), providers);
 
   // List box
@@ -224,12 +249,14 @@ void SettingsDialog::AddPage(Page id, SettingsPage* page, QTreeWidgetItem* paren
   pages_[id] = data;
 }
 
-void SettingsDialog::accept() {
-  // Save settings
+void SettingsDialog::Save() {
   foreach (const PageData& data, pages_.values()) {
     data.page_->Save();
   }
+}
 
+void SettingsDialog::accept() {
+  Save();
   QDialog::accept();
 }
 
@@ -242,6 +269,13 @@ void SettingsDialog::reject() {
   QDialog::reject();
 }
 
+void SettingsDialog::DialogButtonClicked(QAbstractButton* button) {
+  // While we only connect Apply at the moment, this might change in the future
+  if (ui_->buttonBox->button(QDialogButtonBox::Apply) == button) {
+    Save();
+  }
+}
+
 void SettingsDialog::showEvent(QShowEvent* e) {
   // Load settings
   loading_settings_ = true;
