@@ -33,9 +33,11 @@ public:
   // Return false to grey out the page's item in the list.
   virtual bool IsEnabled() const { return true; }
 
-  // Load is called when the dialog is shown, Save when the user clicks OK.
+  // Load is called when the dialog is shown, Save when the user clicks OK, and
+  // Cancel when the user clicks on Cancel
   virtual void Load() = 0;
   virtual void Save() = 0;
+  virtual void Cancel() {}
 
   // The dialog that this page belongs to.
   SettingsDialog* dialog() const { return dialog_; }
