@@ -1,4 +1,5 @@
 /* This file is part of Clementine.
+   Copyright 2010, David Sansome <me@davidsansome.com>
 
    Clementine is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -19,27 +20,37 @@
 
 #include <QWidget>
 
+class BusyIndicator;
 class TaskManager;
-class Ui_MultiLoadingIndicator;
 
 class MultiLoadingIndicator : public QWidget {
   Q_OBJECT
 
- public:
+public:
   MultiLoadingIndicator(QWidget* parent = 0);
-  ~MultiLoadingIndicator();
+
+  static const int kVerticalPadding;
+  static const int kHorizontalPadding;
+  static const int kSpacing;
 
   void SetTaskManager(TaskManager* task_manager);
 
- signals:
+  QSize sizeHint() const;
+
+signals:
   void TaskCountChange(int tasks);
 
- private slots:
+protected:
+  void paintEvent(QPaintEvent*);
+
+private slots:
   void UpdateText();
 
- private:
-  Ui_MultiLoadingIndicator* ui_;
+private:
   TaskManager* task_manager_;
+
+  BusyIndicator* spinner_;
+  QString text_;
 };
 
 #endif // MULTILOADINGINDICATOR_H
