@@ -24,11 +24,10 @@
 
 #include "config.h"
 #include "engines/engine_fwd.h"
-#include "core/backgroundthread.h"
 #include "core/song.h"
-#include "covers/albumcoverloader.h"
 #include "playlist/playlistsequence.h"
 
+class Application;
 class OrgFreedesktopNotificationsInterface;
 class OSDPretty;
 class SystemTrayIcon;
@@ -47,7 +46,7 @@ class OSD : public QObject {
   Q_OBJECT
 
  public:
-  OSD(SystemTrayIcon* tray_icon, QObject* parent = 0);
+  OSD(SystemTrayIcon* tray_icon, Application* app, QObject* parent = 0);
   ~OSD();
 
   static const char* kSettingsGroup;
@@ -70,8 +69,6 @@ class OSD : public QObject {
  public slots:
   void ReloadSettings();
 
-  void ForceShowNextNotification() { force_show_next_ = true; }
-  void SongChanged(const Song& song);
   void Paused();
   void Stopped();
   void PlaylistFinished();
@@ -80,6 +77,8 @@ class OSD : public QObject {
   void RepeatModeChanged(PlaylistSequence::RepeatMode mode);
   void ShuffleModeChanged(PlaylistSequence::ShuffleMode mode);
 
+  void ReshowCurrentSong();
+
 #ifdef HAVE_WIIMOTEDEV
   void WiiremoteActived(int id);
   void WiiremoteDeactived(int id);
@@ -92,12 +91,6 @@ class OSD : public QObject {
   void ShowPreview(const Behaviour type, const QString& line1, const QString& line2, const Song& song);
 
  private:
-  struct WaitingForAlbumArt {
-    QString summary;
-    QString message;
-    QString icon;
-  };
-
   void ShowMessage(const QString& summary,
                    const QString& message = QString(),
                    const QString& icon = QString(),
@@ -113,13 +106,11 @@ class OSD : public QObject {
 
  private slots:
   void CallFinished(QDBusPendingCallWatcher* watcher);
-  void CoverLoaderInitialised();
-  void CoverArtPathReady(const Song& song, const QString& image_path);
-  void AlbumArtLoaded(quint64 id, const QImage& image);
-  void AlbumArtLoaded(const WaitingForAlbumArt info, const QImage& image);
+  void AlbumArtLoaded(const Song& song, const QString& uri, const QImage& image);
 
  private:
   SystemTrayIcon* tray_icon_;
+  Application* app_;
   int timeout_msec_;
   Behaviour behaviour_;
   bool show_on_volume_change_;
@@ -135,8 +126,9 @@ class OSD : public QObject {
 
   OSDPretty* pretty_popup_;
 
-  BackgroundThread<AlbumCoverLoader>* cover_loader_;
-  QMap<quint64, WaitingForAlbumArt> waiting_for_album_art_;
+  Song last_song_;
+  QString last_image_uri_;
+  QImage last_image_;
 
 #ifdef Q_OS_DARWIN
   class GrowlNotificationWrapper;
