@@ -22,6 +22,8 @@
 
 class QLabel;
 
+class Application;
+class MoodbarProxyStyle;
 class Ui_TrackSlider;
 
 class TrackSlider : public QWidget {
@@ -31,22 +33,30 @@ class TrackSlider : public QWidget {
   TrackSlider(QWidget* parent = 0);
   ~TrackSlider();
 
+  void SetApplication(Application* app);
+
   // QWidget
   QSize sizeHint() const;
 
   // QObject
   bool event(QEvent *);
 
+  MoodbarProxyStyle* moodbar_style() const { return moodbar_style_; }
+
+  static const char* kSettingsGroup;
+
  public slots:
   void SetValue(int elapsed, int total);
   void SetStopped();
   void SetCanSeek(bool can_seek);
+  void Seek(int gap);
 
  signals:
   void ValueChanged(int value);
 
  private slots:
   void ValueMaybeChanged(int value);
+  void ToggleTimeDisplay();
 
  private:
   void UpdateTimes(int elapsed);
@@ -56,7 +66,11 @@ class TrackSlider : public QWidget {
  private:
   Ui_TrackSlider* ui_;
 
+  MoodbarProxyStyle* moodbar_style_;
+
   bool setting_value_;
+  bool show_remaining_time_;
+  int slider_maximum_value_; //we cache it to avoid unnecessary updates
 };
 
 #endif // TRACKSLIDER_H
