@@ -34,10 +34,14 @@ public slots:
 
 protected:
   void paintEvent(QPaintEvent*);
+  bool eventFilter(QObject* obj, QEvent* event);
 
 private slots:
   void FadeFinished();
 
+private:
+  void CaptureParent();
+
 private:
   static const int kLoadingPadding;
   static const int kLoadingBorderRadius;
