@@ -18,7 +18,6 @@
 #include "gmock/gmock-matchers.h"
 #include "gtest/gtest.h"
 #include "test_utils.h"
-#include "mock_taglib.h"
 
 #include "core/timeconstants.h"
 #include "playlistparsers/m3uparser.h"
@@ -30,16 +29,11 @@ using ::testing::HasSubstr;
 
 class M3UParserTest : public ::testing::Test {
  protected:
-  static void SetUpTestCase() {
-    testing::DefaultValue<TagLib::String>::Set("foobarbaz");
-  }
-
   M3UParserTest()
       : parser_(NULL) {
   }
 
   M3UParser parser_;
-  MockFileRefFactory taglib_;
 };
 
 TEST_F(M3UParserTest, ParsesMetadata) {
